import type AtpAgent from "@atproto/api";
import { StepBuilder, StepSavePath, StepTapInterceptor, StepWhenPredicate, type StepBuilderConfig } from "../trotsky";
import { Resolvable } from "./utils/resolvable";
/**
 * Represents an abstract step in a sequence managed by the {@link Trotsky} framework.
 * @typeParam P - Type of the parent object.
 * @typeParam C - Type of the context object.
 * @typeParam O - Type of the output object.
 * @public
 */
export declare abstract class Step<P = StepBuilder, C = unknown, O = unknown> extends StepBuilder {
    /** @internal */
    _parent: P;
    /** @internal */
    _context: C | null;
    /** @internal */
    _output: O | null;
    constructor(agent: AtpAgent, parent: P);
    /**
     * Sets the parent.
     * @param context - The parent object.
     * @returns The current {@link Trotsky} instance.
     */
    withParent(parent: StepBuilder): this;
    /**
     * Sets the context.
     * @param context - The context object.
     * @returns The current {@link Trotsky} instance.
     */
    withContext(context: C): this;
    /**
     * Sets the output.
     * @param output - The output object.
     * @returns The current {@link Trotsky} instance.
     */
    withOutput(output: O): this;
    /**
     * Adds a {@link StepWait} step.
     * @param duration - The duration to wait, in milliseconds.
     * @returns The current {@link Step} instance.
     */
    wait(duration?: number): this;
    /**
     *Adds a {@link StepWhen} step.
      * @param predicate - The predicate function.
      * @returns The current {@link Step} instance.
      */
    when(predicate: Resolvable<StepWhenPredicate>): this;
    /**
     * Adds a {@link StepTap} step.
     * @returns The current {@link Step} instance.
     */
    tap(interceptor: StepTapInterceptor): this;
    /**
     * Adds a {@link StepSave} step.
     * @param path - The path of the JSON file to save the output. If not provided, the file path will be created using the current step name.
     * @returns The current {@link Step} instance.
     */
    save(path?: StepSavePath): this;
    /**
     * Retreive the configuration value for a given key on the current
     * step or inherite the value from the parent step.
     * @param key - The configuration key.
     * @returns The configuration value.
     */
    config(key: string): unknown;
    /**
     * Updates the configuration value for a given key on the current step.
     * @param key - The configuration key.
     * @param value - The new value.
     * @returns The current {@link Step} instance.
     */
    config(key: string, value: unknown): this;
    /**
     * Updates the configuration object on the current step.
     * @param config - The new configuration object.
     * @returns The current {@link Step} instance.
     */
    config(config: StepBuilderConfig): this;
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the constructor.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Retrieves the parent object of this instance.
     * @returns The parent object.
     */
    back(): P;
    /**
     * Ends the sequence of steps.
     * @returns The top-level {@link Trotsky} instance.
     */
    end(): StepBuilder;
    /**
     * Abstract method that must be implemented by subclasses to define the behavior of the step.
     * @returns A promise that resolves when the step is applied.
     */
    abstract apply(): Promise<void>;
    /**
     * Runs all steps in the sequence and returns the current {@link Step} instance.
     */
    runHere(): Promise<this>;
    /**
     * Checks if this instance is part of a {@link StepBuilderList}.
     */
    get isStepBuilderListEntry(): boolean;
    get isStepWhen(): boolean;
    /**
     * Retrieves or sets the current context.
     */
    get context(): C;
    set context(value: C);
    /**
     * Retrieves or sets the current output.
     */
    get output(): O | null;
    set output(value: O | null);
}
//# sourceMappingURL=Step.d.ts.map