"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Step = void 0;
const trotsky_1 = require("../trotsky");
/**
 * Represents an abstract step in a sequence managed by the {@link Trotsky} framework.
 * @typeParam P - Type of the parent object.
 * @typeParam C - Type of the context object.
 * @typeParam O - Type of the output object.
 * @public
 */
class Step extends trotsky_1.StepBuilder {
    /** @internal */
    _parent;
    /** @internal */
    _context = null;
    /** @internal */
    _output = null;
    constructor(agent, parent) {
        super(agent);
        this._parent = parent;
    }
    /**
     * Sets the parent.
     * @param context - The parent object.
     * @returns The current {@link Trotsky} instance.
     */
    withParent(parent) {
        this._parent = parent;
        return this;
    }
    /**
     * Sets the context.
     * @param context - The context object.
     * @returns The current {@link Trotsky} instance.
     */
    withContext(context) {
        this._context = context;
        return this;
    }
    /**
     * Sets the output.
     * @param output - The output object.
     * @returns The current {@link Trotsky} instance.
     */
    withOutput(output) {
        this._output = output;
        return this;
    }
    /**
     * Adds a {@link StepWait} step.
     * @param duration - The duration to wait, in milliseconds.
     * @returns The current {@link Step} instance.
     */
    wait(duration = 0) {
        this.append((trotsky_1.StepWait), duration);
        return this;
    }
    /**
     *Adds a {@link StepWhen} step.
      * @param predicate - The predicate function.
      * @returns The current {@link Step} instance.
      */
    when(predicate) {
        this.append((trotsky_1.StepWhen), predicate);
        return this;
    }
    /**
     * Adds a {@link StepTap} step.
     * @returns The current {@link Step} instance.
     */
    tap(interceptor) {
        this.append((trotsky_1.StepTap), interceptor);
        return this;
    }
    /**
     * Adds a {@link StepSave} step.
     * @param path - The path of the JSON file to save the output. If not provided, the file path will be created using the current step name.
     * @returns The current {@link Step} instance.
     */
    save(path = null) {
        this.append((trotsky_1.StepSave), path);
        return this;
    }
    /**
     * Retrieves or updates the configuration value for a given key.
     * @param keyOrconfig - The configuration key or object.
     * @param value - The new value.
     * @returns The current {@link Step} instance if updating, or the configuration value if retrieving.
     * @remarks If the first argument is an object, it will be merged with the existing configuration.
     */
    config(keyOrconfig, value) {
        if (keyOrconfig instanceof Object) {
            return super.config(keyOrconfig);
        }
        if (value !== undefined) {
            return super.config(keyOrconfig, value);
        }
        return this._config[keyOrconfig] ?? this.back().config(keyOrconfig);
    }
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the constructor.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest) {
        return super.clone(this._parent, ...rest);
    }
    /**
     * Retrieves the parent object of this instance.
     * @returns The parent object.
     */
    back() {
        return this._parent;
    }
    /**
     * Ends the sequence of steps.
     * @returns The top-level {@link Trotsky} instance.
     */
    end() {
        if (this.isTrotsky || !this._parent) {
            return this;
        }
        if (this.back().isTrotsky) {
            return this.back();
        }
        return this.back().end();
    }
    /**
     * Runs all steps in the sequence and returns the current {@link Step} instance.
     */
    async runHere() {
        await this.run();
        return this;
    }
    /**
     * Checks if this instance is part of a {@link StepBuilderList}.
     */
    get isStepBuilderListEntry() {
        return this._parent instanceof trotsky_1.StepBuilderList;
    }
    get isStepWhen() {
        return this instanceof trotsky_1.StepWhen;
    }
    /**
     * Retrieves or sets the current context.
     */
    get context() {
        return this._context ?? this.back()?.output;
    }
    set context(value) {
        this._context = value;
    }
    /**
     * Retrieves or sets the current output.
     */
    get output() {
        return this._output;
    }
    set output(value) {
        this._output = value;
    }
}
exports.Step = Step;
//# sourceMappingURL=Step.js.map