"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepActor = void 0;
const ActorMixins_1 = require("./mixins/ActorMixins");
const resolvable_1 = require("./utils/resolvable");
/**
 * Represents a step for retrieving and handling an actor's profile using the Bluesky API.
 * @typeParam P - Type of the parent object.
 * @typeParam C - Type of the context object.
 * @typeParam O - Type of the output object, extending {@link StepActorOutput}.
 * @public
 */
class StepActor extends ActorMixins_1.ActorMixins {
    /** @internal */
    _param;
    /**
     * Initializes a new {@link StepActor} instance.
     * @param agent - The {@link AtpAgent} instance for API interactions.
     * @param parent - The parent object associated with this step.
     * @param param - The parameter representing the actor to retrieve, which can be a resolvable value.
     */
    constructor(agent, parent, param) {
        super(agent, parent);
        this._param = param;
    }
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest) {
        return super.clone(this._param, ...rest);
    }
    /**
     * Applies the step by resolving the actor parameter and fetching the actor's profile.
     * Sets the profile data as the output of this step.
     * @returns A promise that resolves when the step is complete.
     */
    async apply() {
        const actor = await (0, resolvable_1.resolveValue)(this, this._param);
        const { data } = await this.agent.getProfile({ actor });
        this.output = data;
    }
}
exports.StepActor = StepActor;
//# sourceMappingURL=StepActor.js.map