"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepActorUnfollow = void 0;
const matchProperty_1 = __importDefault(require("./utils/matchProperty"));
const trotsky_1 = require("../trotsky");
/**
 * Represents step that unfollows the current actor (if followed).
 *
 * @typeParam P - The parent step type, defaulting to {@link StepActor}.
 * @typeParam C - The context or child output type, defaulting to {@link StepActorOutput}.
 * @typeParam O - The output type produced by this step, defaulting to `null`.
 * @public
 */
class StepActorUnfollow extends trotsky_1.Step {
    /**
     * Applies the unfollow logic:
     * 1. Retrieves the current actor's relationship with the specified context actor.
     * 2. Checks if the current actor is indeed following the specified context actor.
     * 3. If so, calls the API to remove the follow relationship.
     *
     * @throws Error
     * If no context is available for the step, indicating the actor is unknown.
     */
    async apply() {
        if (!this.context) {
            throw new Error("No context found for StepActorUnfollow");
        }
        const actor = this.agent.did;
        const others = [this.context.did];
        const { "data": { relationships } } = await this.agent.app.bsky.graph.getRelationships({ actor, others });
        const relationship = relationships.find((0, matchProperty_1.default)("did", this.context.did));
        // if the relationship does not exist, there is nothing to do
        if (!relationship?.following)
            return;
        await this.agent.deleteFollow(relationship.following);
    }
}
exports.StepActorUnfollow = StepActorUnfollow;
//# sourceMappingURL=StepActorUnfollow.js.map