"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepActors = void 0;
const trotsky_1 = require("../trotsky");
const resolvable_1 = require("./utils/resolvable");
/**
 * An abstract class that extends {@link StepBuilderList} to handle a list of actor profiles.
 * It defines a collection of steps dealing with actor entries (`StepActorsEntry`),
 * which can be appended via the `each()` method.
 *
 * @typeParam P - The parent step builder type, defaults to `StepBuilder`.
 * @typeParam C - The context type, defaults to `null`.
 * @typeParam O - The output type, extending {@link StepActorsOutput}, defaults to `StepActorsOutput`.
 * @public
 */
class StepActors extends trotsky_1.StepBuilderList {
    /**
     * Stores the list of appended actor entries.
     * @internal
     */
    _steps = [];
    /**
     * The parameter passed to the {@link StepActors} class to retreive the actors
     * @internal
     */
    _param;
    /**
     * Initializes a new {@link StepActors} instance.
     * @param agent - The {@link AtpAgent} instance for API interactions.
     * @param parent - The parent object associated with this step.
     * @param param - The parameter representing the actors to retrieve, which can be a resolvable value.
     */
    constructor(agent, parent, param) {
        super(agent, parent);
        this._param = param;
    }
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest) {
        return super.clone(this._param, ...rest);
    }
    /**
     * Appends a new {@link StepActorsEntry} instance to the step list
     * and returns the newly created entry.
     *
     * @returns A new step entry configured for actor processing.
     */
    each(iterator) {
        return this.withIterator(iterator).append((trotsky_1.StepActorsEntry));
    }
    /**
     * Applies the step by resolving the actor parameter and fetching the actors' profiles.
     * Sets the profile data as the output of this step.
     * @returns A promise that resolves when the step is complete.
     */
    async applyPagination() {
        const actors = await (0, resolvable_1.resolveValue)(this, this._param);
        const { data } = await this.agent.app.bsky.actor.getProfiles({ actors });
        this.output = data.profiles;
    }
}
exports.StepActors = StepActors;
//# sourceMappingURL=StepActors.js.map