import type AtpAgent from "@atproto/api";
import { Step } from "../trotsky";
/**
 * Represents the configuration object for a {@link StepBuilder} instance.
 * @public
 */
export type StepBuilderConfig = object;
/**
 * Represents a builder for a sequence of steps. This class is used internally by the {@link Trotsky} framework.
 */
export declare abstract class StepBuilder {
    /** @internal */
    _agent: AtpAgent;
    /** @internal */
    _steps: Step<this>[];
    /** @internal */
    _config: StepBuilderConfig;
    /**
     * Initializes a new {@link StepBuilder} instance.
     * @param agent - The {@link AtpAgent} instance for API interactions.
     */
    constructor(agent: AtpAgent);
    /**
     * Updates the agent instance.
     * @param agent - The new {@link AtpAgent} instance.
     *
     * @returns The current {@link Trotsky} instance.
     */
    withAgent(agent: AtpAgent): this;
    /**
     * Retreive the configuration value for a given key.
     * @param key - The configuration key.
     * @returns The configuration value.
     */
    config(key: string): unknown;
    /**
     * Updates the configuration value for a given key.
     * @param key - The configuration key.
     * @param value - The new value.
     * @returns The current {@link StepBuilder} instance.
     */
    config(key: string, value: unknown): this;
    /**
     * Updates the configuration object.
     * @param config - The new configuration object.
     * @returns The current {@link StepBuilder} instance.
     */
    config(config: StepBuilderConfig): this;
    /**
     * Clones the current instance, including all steps and configuration.
     *
     * @param rest - Additional arguments to pass to the constructor.
     * @returns A new instance of the current class.
     */
    clone(...rest: unknown[]): this;
    /**
     * Clears all steps from the sequence.
     * @returns The current {@link StepBuilder} instance.
     */
    clear(): this;
    /**
     * Appends a new step to the sequence.
     * @param type - The step class.
     * @param args - Arguments for the step.
     * @returns The new step instance.
     */
    append<Type extends Step<this>>(type: new (agent: AtpAgent, parent: this, ...args: any[]) => Type, ...args: unknown[]): Type;
    /**
     * Adds a step to the sequence.
     * @param stepOrIndex - The step instance or index to remove.
     * @returns The current {@link StepBuilder} instance.
     */
    slice(stepOrIndex: Step<this> | number): this;
    /**
     * Pushes a step to the sequence.
     * @param step - The step instance.
     * @returns The current {@link StepBuilder} instance.
     */
    push(...steps: Step<this>[]): this;
    /**
     * Runs the sequence of steps, returning the top-level {@link StepBuilder} instance.
     */
    run(): any;
    /**
     * Runs all steps in the sequence.
     * @returns The current {@link Trotsky} instance.
     */
    runAll(): Promise<this>;
    /**
     * Schedules the sequence to run at a given interval.
     * @param expression - The cron {@link https://github.com/kelektiv/node-cron | expression} defining the schedule.
     * @returns A cron job instance.
     */
    schedule(expression: string): any;
    /**
     * Applies all steps in the sequence.
     */
    applyAll(): Promise<void>;
    /**
     * Checks if this is a top-level {@link Trotsky} instance.
     */
    get isTrotsky(): boolean;
    /**
     * Retrieves all steps in the sequence.
     */
    get steps(): Step[];
    /**
     * Retrieves a flattened list of all steps, including nested ones.
     */
    get flattenSteps(): Step[];
    /**
     * Retrieves the current {@link AtpAgent} instance.
     */
    get agent(): AtpAgent;
}
//# sourceMappingURL=StepBuilder.d.ts.map