"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepBuilder = void 0;
const node_cron_1 = __importDefault(require("node-cron"));
const trotsky_1 = __importStar(require("../trotsky"));
/**
 * Represents a builder for a sequence of steps. This class is used internally by the {@link Trotsky} framework.
 */
class StepBuilder {
    /** @internal */
    _agent;
    /** @internal */
    _steps = [];
    /** @internal */
    _config = {};
    /**
     * Initializes a new {@link StepBuilder} instance.
     * @param agent - The {@link AtpAgent} instance for API interactions.
     */
    constructor(agent) {
        this._agent = agent;
    }
    /**
     * Updates the agent instance.
     * @param agent - The new {@link AtpAgent} instance.
     *
     * @returns The current {@link Trotsky} instance.
     */
    withAgent(agent) {
        this._agent = agent;
        return this;
    }
    /**
     * Retrieves or updates the configuration value for a given key.
     * @param keyOrconfig - The configuration key or object.
     * @param value - The new value.
     * @returns The current {@link StepBuilder} instance if updating, or the configuration value if retrieving.
     * @remarks If the first argument is an object, it will be merged with the existing configuration.
     */
    config(keyOrconfig, value) {
        if (keyOrconfig instanceof Object) {
            this._config = { ...this._config, ...keyOrconfig };
            return this;
        }
        if (value !== undefined) {
            this._config[keyOrconfig] = value;
            return this;
        }
        return this._config[keyOrconfig];
    }
    /**
     * Clones the current instance, including all steps and configuration.
     *
     * @param rest - Additional arguments to pass to the constructor.
     * @returns A new instance of the current class.
     */
    clone(...rest) {
        const Constructor = this.constructor;
        const clone = new Constructor(this.agent, ...rest);
        // Then, clone the child steps
        for (const step of this.steps) {
            // When cloning the child step, we must also set its parent
            // or its going to use the original parent of the step.
            clone.push(step.clone().withParent(clone));
        }
        // Finally, clone the config (and return the new instance)
        return clone.config({ ...this._config });
    }
    /**
     * Clears all steps from the sequence.
     * @returns The current {@link StepBuilder} instance.
     */
    clear() {
        this._steps = [];
        return this;
    }
    /**
     * Appends a new step to the sequence.
     * @param type - The step class.
     * @param args - Arguments for the step.
     * @returns The new step instance.
     */
    append(type, ...args) {
        const step = new type(this.agent, this, ...args);
        this.push(step);
        return step;
    }
    /**
     * Adds a step to the sequence.
     * @param stepOrIndex - The step instance or index to remove.
     * @returns The current {@link StepBuilder} instance.
     */
    slice(stepOrIndex) {
        this._steps = this._steps.filter((step, index) => {
            return stepOrIndex instanceof trotsky_1.Step ? step !== stepOrIndex : index !== stepOrIndex;
        });
        return this;
    }
    /**
     * Pushes a step to the sequence.
     * @param step - The step instance.
     * @returns The current {@link StepBuilder} instance.
     */
    push(...steps) {
        for (const step of steps) {
            this._steps.push(step);
        }
        return this;
    }
    /**
     * Runs the sequence of steps, returning the top-level {@link StepBuilder} instance.
     */
    async run() {
        if (this.isTrotsky) {
            return this.runAll();
        }
        // If the current instance is not a top-level Trotsky instance, 
        // we assume it's a nested Step instance and run the sequence from the top.
        return this.end().run();
    }
    /**
     * Runs all steps in the sequence.
     * @returns The current {@link Trotsky} instance.
     */
    async runAll() {
        await this.applyAll();
        return this;
    }
    /**
     * Schedules the sequence to run at a given interval.
     * @param expression - The cron {@link https://github.com/kelektiv/node-cron | expression} defining the schedule.
     * @returns A cron job instance.
     */
    schedule(expression) {
        return node_cron_1.default.schedule(expression, this.run.bind(this));
    }
    /**
     * Applies all steps in the sequence.
     */
    async applyAll() {
        for (const step of this.steps) {
            await step.apply();
            // Skip the rest of the steps if the current step is a StepWhen and its output is falsy
            if (step.isStepWhen && !step.output)
                break;
            if (!step.isStepBuilderListEntry) {
                await step.applyAll();
            }
        }
    }
    /**
     * Checks if this is a top-level {@link Trotsky} instance.
     */
    get isTrotsky() {
        return Object.getPrototypeOf(this) === trotsky_1.default.prototype;
    }
    /**
     * Retrieves all steps in the sequence.
     */
    get steps() {
        return this._steps;
    }
    /**
     * Retrieves a flattened list of all steps, including nested ones.
     */
    get flattenSteps() {
        return this.steps.reduce((acc, step) => {
            return [...acc, step, ...step.flattenSteps];
        }, []);
    }
    /**
     * Retrieves the current {@link AtpAgent} instance.
     */
    get agent() {
        return this._agent;
    }
}
exports.StepBuilder = StepBuilder;
//# sourceMappingURL=StepBuilder.js.map