"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepCreatePost = void 0;
const trotsky_1 = require("../trotsky");
const resolvable_1 = require("./utils/resolvable");
/**
 * Represents step that creates a new post on Bluesky using the specified record parameters.
 *
 * @typeParam P - The parent step type, defaulting to {@link StepBuilder}.
 * @typeParam C - The child context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to {@link StepCreatePostOutput}.
 * @public
 */
class StepCreatePost extends trotsky_1.Step {
    /**
     * Stores the parameters needed to create a post, which may be resolved at runtime.
     */
    _record;
    /**
     * Initializes the create-post step with the provided agent, parent, and record parameters.
     * @param agent - The AT protocol agent for making API calls.
     * @param parent - The parent step in the chain.
     * @param record - The post record parameters, possibly resolvable at runtime.
     */
    constructor(agent, parent, record) {
        super(agent, parent);
        this._record = record;
    }
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest) {
        return super.clone(this._record, ...rest);
    }
    /**
     * Executes the create-post step by sending a request to the Bluesky service
     * and assigning the returned URI and CID to `this.output`.
     */
    async apply() {
        this.output = await this.agent.post(await this.queryParams());
    }
    /**
     * Resolves the post creation parameters from a potentially dynamic input source.
     * @returns A promise that resolves to the fully populated {@link StepCreatePostParams}.
     */
    async queryParams() {
        return (0, resolvable_1.resolveValue)(this, this._record);
    }
}
exports.StepCreatePost = StepCreatePost;
//# sourceMappingURL=StepCreatePost.js.map