import type { AtUri, AtpAgent, AppBskyGraphGetList, AppBskyGraphDefs } from "@atproto/api";
import { ListMixins, type StepBuilder } from "../trotsky";
import type { Resolvable } from "./utils/resolvable";
/**
 * Defines the query parameters for retrieving a list.
 * @public
 */
export type StepListQueryParams = AppBskyGraphGetList.QueryParams;
/**
 * Represents a list's URI, which can be a string or an {@link AtUri}.
 * @public
 */
export type StepListUri = string | AtUri;
/**
 * Represents the output of a retrieved list, including its URI and CID
 */
export type StepListOutput = AppBskyGraphDefs.ListView;
/**
 * Represents a step for retrieving a list by its URI.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepBuilder}.
 * @typeParam C - The child context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to {@link StepListOutput}.
 * @public
 */
export declare class StepList<P = StepBuilder, C = null, O extends StepListOutput = StepListOutput> extends ListMixins<P, C, O> {
    /**
     * The URI of the list to retrieve, which can be resolved dynamically at runtime.
     * @internal
     */
    _uri: Resolvable<StepListUri>;
    /**
     * Initializes the StepList instance with the given agent, parent, and URI.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param uri - The URI of the list to retrieve, possibly resolvable at runtime.
     */
    constructor(agent: AtpAgent, parent: P, uri: Resolvable<StepListUri>);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Applies the step logic to retrieve the list and sets the output to the retrieved list's data.
     * If no lists are found, the output is set to `null`.
     *
     * @override
     */
    apply(): Promise<void>;
    /**
     * Resolves the query parameters for retrieving the list, including its URI.
     *
     * @returns A promise that resolves to the query parameters.
     */
    queryParams(): Promise<StepListQueryParams>;
}
//# sourceMappingURL=StepList.d.ts.map