"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepList = void 0;
const trotsky_1 = require("../trotsky");
const resolvable_1 = require("./utils/resolvable");
/**
 * Represents a step for retrieving a list by its URI.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepBuilder}.
 * @typeParam C - The child context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to {@link StepListOutput}.
 * @public
 */
class StepList extends trotsky_1.ListMixins {
    /**
     * The URI of the list to retrieve, which can be resolved dynamically at runtime.
     * @internal
     */
    _uri;
    /**
     * Initializes the StepList instance with the given agent, parent, and URI.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param uri - The URI of the list to retrieve, possibly resolvable at runtime.
     */
    constructor(agent, parent, uri) {
        super(agent, parent);
        this._uri = uri;
    }
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest) {
        return super.clone(this._uri, ...rest);
    }
    /**
     * Applies the step logic to retrieve the list and sets the output to the retrieved list's data.
     * If no lists are found, the output is set to `null`.
     *
     * @override
     */
    async apply() {
        const { "data": { list } } = await this.agent.app.bsky.graph.getList(await this.queryParams());
        this.output = list ?? null;
    }
    /**
     * Resolves the query parameters for retrieving the list, including its URI.
     *
     * @returns A promise that resolves to the query parameters.
     */
    async queryParams() {
        const uri = await (0, resolvable_1.resolveValue)(this, this._uri);
        const list = uri.toString();
        return { list, "limit": 1 };
    }
}
exports.StepList = StepList;
//# sourceMappingURL=StepList.js.map