"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepPost = void 0;
const PostMixins_1 = require("./mixins/PostMixins");
const resolvable_1 = require("./utils/resolvable");
/**
 * Represents a step for retrieving a post by its URI.
 * Extends {@link PostMixins} for post-related operations.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepBuilder}.
 * @typeParam C - The child context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to {@link StepPostOutput}.
 * @public
 */
class StepPost extends PostMixins_1.PostMixins {
    /**
     * The URI of the post to retrieve, which can be resolved dynamically at runtime.
     */
    _uri;
    /**
     * Initializes the StepPost instance with the given agent, parent, and URI.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param uri - The URI of the post to retrieve, possibly resolvable at runtime.
     */
    constructor(agent, parent, uri) {
        super(agent, parent);
        this._uri = uri;
    }
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest) {
        return super.clone(this._uri, ...rest);
    }
    /**
     * Applies the step logic to retrieve the post and sets the output to the retrieved post's data.
     * If no posts are found, the output is set to `null`.
     *
     * @override
     */
    async apply() {
        const { "data": { posts } } = await this.agent.getPosts(await this.queryParams());
        this.output = (posts.length ? posts[posts.length - 1] : null);
    }
    /**
     * Resolves the query parameters for retrieving the post, including its URI.
     *
     * @returns A promise that resolves to the query parameters.
     */
    async queryParams() {
        const uri = await (0, resolvable_1.resolveValue)(this, this._uri);
        const uris = [uri.toString?.() ?? uri];
        return { uris };
    }
}
exports.StepPost = StepPost;
//# sourceMappingURL=StepPost.js.map