import type { AppBskyFeedPost, AtpAgent } from "@atproto/api";
import type { ResolvablePostReplyParams } from "./mixins/PostMixins";
import { Step, type StepPost, type StepPostOutput } from "../trotsky";
/**
 * Represents a reference to a post record, including its URI and CID.
 * @public
 */
export interface StepPostReplyRecordRef {
    "uri": string;
    "cid": string;
}
/**
 * Represents the reply structure, containing references to both the parent and root posts.
 * @public
 */
export interface StepPostReplyRef {
    "parent": StepPostReplyRecordRef;
    "root": StepPostReplyRecordRef;
}
/**
 * Represents the output of a post reply step, including the URI and CID of the reply.
 * @public
 */
export type StepPostReplyOutput = StepPostReplyRecordRef;
/**
 * Represents a step for replying to a specific post.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepPost}.
 * @typeParam C - The context type, defaulting to {@link StepPostOutput}.
 * @typeParam O - The output type, defaulting to {@link StepPostReplyOutput}.
 * @public
 */
export declare class StepPostReply<P = StepPost, C extends StepPostOutput = StepPostOutput, O extends StepPostReplyOutput = StepPostReplyOutput> extends Step<P, C, O> {
    /**
     * The record containing the reply parameters, which can be resolved dynamically at runtime.
     */
    _record: ResolvablePostReplyParams;
    /**
     * Initializes the StepPostReply instance with the given agent, parent, and reply record parameters.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param record - The reply parameters, possibly resolvable at runtime.
     */
    constructor(agent: AtpAgent, parent: P, record: ResolvablePostReplyParams);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Applies the step logic to create a reply to the post provided in the context.
     * The resulting URI and CID of the reply are stored in the output.
     *
     * @override
     */
    apply(): Promise<void>;
    /**
     * Resolves the query parameters for the reply, including reply references and creation time.
     *
     * @returns A promise that resolves to the complete reply record.
     */
    queryParams(): Promise<AppBskyFeedPost.Record>;
    /**
     * Constructs the reply parameters for the post, including parent and root references.
     *
     * @returns An object containing the root and parent references for the reply.
     */
    replyParams(): AppBskyFeedPost.Record["reply"];
}
//# sourceMappingURL=StepPostReply.d.ts.map