"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepPostReply = void 0;
const resolvable_1 = require("./utils/resolvable");
const trotsky_1 = require("../trotsky");
/**
 * Represents a step for replying to a specific post.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepPost}.
 * @typeParam C - The context type, defaulting to {@link StepPostOutput}.
 * @typeParam O - The output type, defaulting to {@link StepPostReplyOutput}.
 * @public
 */
class StepPostReply extends trotsky_1.Step {
    /**
     * The record containing the reply parameters, which can be resolved dynamically at runtime.
     */
    _record;
    /**
     * Initializes the StepPostReply instance with the given agent, parent, and reply record parameters.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param record - The reply parameters, possibly resolvable at runtime.
     */
    constructor(agent, parent, record) {
        super(agent, parent);
        this._record = record;
    }
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest) {
        return super.clone(this._record, ...rest);
    }
    /**
     * Applies the step logic to create a reply to the post provided in the context.
     * The resulting URI and CID of the reply are stored in the output.
     *
     * @override
     */
    async apply() {
        this.output = await this.agent.post(await this.queryParams());
    }
    /**
     * Resolves the query parameters for the reply, including reply references and creation time.
     *
     * @returns A promise that resolves to the complete reply record.
     */
    async queryParams() {
        const params = await (0, resolvable_1.resolveValue)(this, this._record);
        const createdAt = new Date().toISOString();
        const reply = this.replyParams();
        return {
            createdAt,
            reply,
            ...params
        };
    }
    /**
     * Constructs the reply parameters for the post, including parent and root references.
     *
     * @returns An object containing the root and parent references for the reply.
     */
    replyParams() {
        const post = this.context;
        const reply = post?.record?.reply ?? null;
        const parent = { "uri": post.uri, "cid": post.cid };
        const root = reply ? { "uri": reply.root.uri, "cid": reply.root.cid } : parent;
        return { root, parent };
    }
}
exports.StepPostReply = StepPostReply;
//# sourceMappingURL=StepPostReply.js.map