"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepPosts = void 0;
const trotsky_1 = require("../trotsky");
const resolvable_1 = require("./utils/resolvable");
/**
 * Abstract class representing a list of posts to process.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepBuilder}.
 * @typeParam C - The child context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to {@link StepPostsOutput}.
 * @public
 */
class StepPosts extends trotsky_1.StepBuilderList {
    /**
     * Holds the list of steps to be executed for each post entry.
     */
    _steps = [];
    /**
     * The URIs of the posts to retrieve, which can be resolved dynamically at runtime.
     */
    _uris;
    /**
     * Initializes the StepPost instance with the given agent, parent, and URI.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param uris - The URIs of the post sto retrieve, possibly resolvable at runtime.
     */
    constructor(agent, parent, uris = []) {
        super(agent, parent);
        this._uris = uris;
    }
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepPosts} instance.
     */
    clone(...rest) {
        return super.clone(this._uris, ...rest);
    }
    /**
     * Appends a new post entry step to the current list and returns it.
     *
     * @param iterator - Optional iterator function to be executed for each item in the list.
     * @returns The newly appended {@link StepPostsEntry} instance.
     */
    each(iterator) {
        return this.withIterator(iterator).append((trotsky_1.StepPostsEntry));
    }
    /**
     * Applies the step by resolving the actor parameter and fetching the actors' profiles.
     * Sets the posts data as the output of this step.
     * @returns A promise that resolves when the step is complete.
     */
    async applyPagination() {
        const uris = (await (0, resolvable_1.resolveValue)(this, this._uris)).map(uri => uri.toString());
        const { data } = await this.agent.app.bsky.feed.getPosts({ uris });
        this.output = data.posts;
    }
}
exports.StepPosts = StepPosts;
//# sourceMappingURL=StepPosts.js.map