"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepSearchPosts = void 0;
const trotsky_1 = require("../trotsky");
/**
 * Represents a step for searching posts on Bluesky, with support for pagination.
 *
 * @typeParam P - The parent type of this step.
 * @typeParam C - The child context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to {@link StepSearchPostsOutput}.
 * @public
 */
class StepSearchPosts extends trotsky_1.StepPosts {
    /**
     * The initial query parameters for the search.
     */
    _queryParams;
    /**
     * Initializes the StepSearchPosts instance with the given agent, parent, and query parameters.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param queryParams - The initial query parameters for the search.
     */
    constructor(agent, parent, queryParams) {
        super(agent, parent);
        this._queryParams = queryParams;
    }
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest) {
        const step = super.clone(...rest);
        step._queryParams = this._queryParams;
        return step;
    }
    /**
     * Applies the pagination logic to retrieve posts based on the search parameters.
     * Results are stored in the `output` property.
     *
     * @override
     */
    async applyPagination() {
        this.output = await this.paginate("posts", (cursor) => {
            return this
                .agent
                .app.bsky.feed
                .searchPosts(this.queryParams(cursor));
        });
    }
    /**
     * Constructs the query parameters for the current pagination step.
     *
     * @param cursor - The cursor indicating the current position in the paginated results.
     * @returns The query parameters for the search.
     */
    queryParams(cursor) {
        return { ...this._queryParams, cursor };
    }
}
exports.StepSearchPosts = StepSearchPosts;
//# sourceMappingURL=StepSearchPosts.js.map