"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepStreamPosts = void 0;
const api_1 = require("@atproto/api");
const trotsky_1 = require("../trotsky");
const jetstream_1 = require("./utils/jetstream");
/**
 * @experimental
 *
 * Represents a streaming step that processes posts in real-time.
 *
 * @typeParam P - The type of the parent constraint.
 * @typeParam C - The type of the context.
 * @typeParam O - The type of the output, defaults to `StepPostOutput`.
 * @public
 *
 * @remarks
 * This class allows for the streaming and processing of posts as they are
 * committed to the network. To achieve this, it utilizes the Bluesky Firehose
 * API to listen for post events and process them in real-time. The stream can
 * use a lot of bandwidth since it processes every post event.
 *
 * @see {@link StepBuilderStream}
 */
class StepStreamPosts extends trotsky_1.StepBuilderStream {
    _steps = [];
    /**
     * Adds a {@link StepPostsEntry} step to the stream.
     *
     * @returns The new {@link StepPostsEntry} instance.
     */
    each() {
        return this.append((trotsky_1.StepPostsEntry));
    }
    /**
     * Builds and returns an event emitter configured for post events.
     *
     * @returns A promise that resolves to the event emitter.
     */
    buildEventEmitter() {
        return (0, jetstream_1.buildEventEmitter)(["app.bsky.feed.post"]);
    }
    /**
     * Resolves the output for a given commit message.
     *
     * @param message - The commit message containing post data.
     * @returns A promise that resolves to the output object.
     */
    async resolveOutput(message) {
        const commit = message.commit;
        const cid = commit.cid;
        const record = commit.record;
        const uri = api_1.AtUri.make(message.did, commit.collection, commit.rkey).toString();
        return { uri, cid, record };
    }
}
exports.StepStreamPosts = StepStreamPosts;
//# sourceMappingURL=StepStreamPosts.js.map