"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepTap = void 0;
const trotsky_1 = require("../trotsky");
/**
 * Represents a step that allows interception of the current {@link Step} instance
 * for custom processing or side effects.
 *
 * @typeParam P - The type of the parent constraint.
 * @typeParam C - The type of the context.
 * @typeParam O - The type of the output, defaults to `null`.
 * @public
 *
 * @remarks
 * The `StepTap` class enables the execution of a provided interceptor function,
 * which receives the current step as its argument. This is useful for debugging,
 * logging, or performing side effects without altering the main flow.
 *
 * @see {@link Step}
 */
class StepTap extends trotsky_1.Step {
    /**
     * The interceptor function to be executed during the apply phase.
     * It receives the current {@link Step} instance and can perform custom operations.
     */
    _interceptor;
    /**
     * Creates an instance of `StepTap`.
     *
     * @param agent - The ATP agent used for API interactions.
     * @param parent - The parent step or context.
     * @param interceptor - A function that intercepts the current step for custom processing.
     */
    constructor(agent, parent, interceptor) {
        super(agent, parent);
        this._interceptor = interceptor;
    }
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest) {
        return super.clone(this._interceptor, ...rest);
    }
    /**
     * Executes the interceptor function with the current step as its argument.
     */
    async apply() {
        await this._interceptor(this);
    }
}
exports.StepTap = StepTap;
//# sourceMappingURL=StepTap.js.map