"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepWhen = void 0;
const trotsky_1 = require("../trotsky");
const resolvable_1 = require("./utils/resolvable");
/**
 * Represents a conditional step that evaluates a predicate to determine its outcome.
 *
 * @typeParam P - The type of the parent constraint.
 * @typeParam C - The type of the context.
 * @typeParam O - The type of the output, defaults to `boolean`.
 * @public
 *
 * @remarks
 * This step evaluates a predicate, which can be a boolean value or a resolvable that
 * resolves to a boolean. The result of the predicate evaluation is stored in the output.
 */
class StepWhen extends trotsky_1.Step {
    _predicate;
    /**
     * Creates an instance of `StepWhen`.
     *
     * @param agent - The ATP agent used for API interactions.
     * @param parent - The parent step or context.
     * @param predicate - A boolean or a resolvable that resolves to a boolean,
     *                    determining the condition for this step.
     */
    constructor(agent, parent, predicate) {
        super(agent, parent);
        this._predicate = predicate;
    }
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest) {
        return super.clone(this._predicate, ...rest);
    }
    /**
     * Evaluates the predicate and sets the output to its result.
     */
    async apply() {
        this.output = (await (0, resolvable_1.resolveValue)(this, this._predicate));
    }
}
exports.StepWhen = StepWhen;
//# sourceMappingURL=StepWhen.js.map