import type { AtpAgent } from "@atproto/api";
import type { QueryParams } from "@atproto/api/src/client/types/app/bsky/feed/searchPosts";
import type { StepActorParam } from "./StepActor";
import type { StepPostUri } from "./StepPost";
import type { StepPostsUris } from "./StepPosts";
import type { StepCreatePostParams } from "./StepCreatePost";
import type { StepWhenPredicate } from "./StepWhen";
import type { StepTapInterceptor } from "./StepTap";
import type { Resolvable } from "./utils/resolvable";
import { StepActor, StepActors, StepPost, StepCreatePost, StepList, StepListUri, StepSearchPosts, StepStreamPosts, StepBuilder, StepActorsParam, StepPosts, StepSavePath } from "../trotsky";
/**
 * Trotsky orchestrates a sequence of steps that interact with an {@link AtpAgent}.
 * @typeParam P - Type of the parent object.
 * @typeParam C - Type of the context object.
 * @typeParam O - Type of the output object.
 * @public
 */
export declare class Trotsky extends StepBuilder {
    /**
     * Adds a {@link StepActor} step.
     * @param param - Parameters for the actor step.
     * @returns The new {@link StepActor} instance.
     */
    actor(param: Resolvable<StepActorParam>): StepActor<this>;
    /**
     * Adds a {@link StepActors} step.
     * @param param - Parameters for the actors step.
     * @returns The new {@link StepActors} instance.
     */
    actors(param: Resolvable<StepActorsParam>): StepActors<this>;
    /**
     * Adds a {@link StepPost} step.
     * @param uri - The post URI.
     * @returns The new {@link StepPost} instance.
     */
    post(uri: Resolvable<StepPostUri>): StepPost<this>;
    /**
     * Adds a {@link StepPosts} step.
     * @param uris - The post URIs.
     * @returns The new {@link StepPosts} instance.
     */
    posts(uris: Resolvable<StepPostsUris>): StepPosts<this>;
    /**
     * Adds a {@link StepCreatePost} step.
     * @param record - Parameters for creating a post.
     * @returns The new {@link StepCreatePost} instance.
     */
    createPost(record: StepCreatePostParams): StepCreatePost<this>;
    /**
     * Adds a {@link StepList} step.
     * @param uri - The list URI.
     * @returns The new {@link StepList} instance.
     */
    list(uri: Resolvable<StepListUri>): StepList<this, null, import("@atproto/api/dist/client/types/app/bsky/graph/defs").ListView>;
    /**
     * Adds a {@link StepSearchPosts} step.
     * @param queryParams - Search parameters.
     * @returns The new {@link StepSearchPosts} instance.
     */
    searchPosts(queryParams: QueryParams): StepSearchPosts<this>;
    /**
     * Adds a {@link StepStreamPosts} step.
     * @returns The new {@link StepStreamPosts} instance.
     */
    streamPosts<T = StepStreamPosts<this>>(): T;
    /**
     * Adds a {@link StepSave} step.
     * @param path - The path of the JSON file to save the output. If not provided, the file path will be created using the current step name.
     * @returns The current {@link Trotsky} instance.
     */
    save(path?: StepSavePath): this;
    /**
     * Adds a {@link StepWait} step.
     * @param duration - The duration to wait, in milliseconds.
     * @returns The current {@link Trotsky} instance.
     */
    wait(duration?: number): this;
    /**
     *Adds a {@link StepWhen} step.
     * @param predicate - The predicate function.
     * @returns The current {@link Trotsky} instance.
     */
    when(predicate: Resolvable<StepWhenPredicate>): this;
    /**
     * Adds a {@link StepTap} step.
     * @returns The current {@link Trotsky} instance.
     */
    tap(interceptor: StepTapInterceptor): this;
    /**
     * Initializes a new {@link Trotsky} instance with the specified agent.
     * @param agent - The {@link AtpAgent} instance.
     * @returns A new {@link Trotsky} instance.
     */
    static init(agent: AtpAgent): Trotsky;
}
//# sourceMappingURL=Trotsky.d.ts.map