"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Trotsky = void 0;
const trotsky_1 = require("../trotsky");
/**
 * Trotsky orchestrates a sequence of steps that interact with an {@link AtpAgent}.
 * @typeParam P - Type of the parent object.
 * @typeParam C - Type of the context object.
 * @typeParam O - Type of the output object.
 * @public
 */
class Trotsky extends trotsky_1.StepBuilder {
    /**
     * Adds a {@link StepActor} step.
     * @param param - Parameters for the actor step.
     * @returns The new {@link StepActor} instance.
     */
    actor(param) {
        return this.append((trotsky_1.StepActor), param);
    }
    /**
     * Adds a {@link StepActors} step.
     * @param param - Parameters for the actors step.
     * @returns The new {@link StepActors} instance.
     */
    actors(param) {
        return this.append((trotsky_1.StepActors), param);
    }
    /**
     * Adds a {@link StepPost} step.
     * @param uri - The post URI.
     * @returns The new {@link StepPost} instance.
     */
    post(uri) {
        return this.append((trotsky_1.StepPost), uri);
    }
    /**
     * Adds a {@link StepPosts} step.
     * @param uris - The post URIs.
     * @returns The new {@link StepPosts} instance.
     */
    posts(uris) {
        return this.append((trotsky_1.StepPosts), uris);
    }
    /**
     * Adds a {@link StepCreatePost} step.
     * @param record - Parameters for creating a post.
     * @returns The new {@link StepCreatePost} instance.
     */
    createPost(record) {
        return this.append((trotsky_1.StepCreatePost), record);
    }
    /**
     * Adds a {@link StepList} step.
     * @param uri - The list URI.
     * @returns The new {@link StepList} instance.
     */
    list(uri) {
        return this.append(trotsky_1.StepList, uri);
    }
    /**
     * Adds a {@link StepSearchPosts} step.
     * @param queryParams - Search parameters.
     * @returns The new {@link StepSearchPosts} instance.
     */
    searchPosts(queryParams) {
        return this.append((trotsky_1.StepSearchPosts), queryParams);
    }
    /**
     * Adds a {@link StepStreamPosts} step.
     * @returns The new {@link StepStreamPosts} instance.
     */
    streamPosts() {
        return this.append((trotsky_1.StepStreamPosts));
    }
    /**
     * Adds a {@link StepSave} step.
     * @param path - The path of the JSON file to save the output. If not provided, the file path will be created using the current step name.
     * @returns The current {@link Trotsky} instance.
     */
    save(path = null) {
        this.append((trotsky_1.StepSave), path);
        return this;
    }
    /**
     * Adds a {@link StepWait} step.
     * @param duration - The duration to wait, in milliseconds.
     * @returns The current {@link Trotsky} instance.
     */
    wait(duration = 0) {
        this.append((trotsky_1.StepWait), duration);
        return this;
    }
    /**
     *Adds a {@link StepWhen} step.
     * @param predicate - The predicate function.
     * @returns The current {@link Trotsky} instance.
     */
    when(predicate) {
        this.append((trotsky_1.StepWhen), predicate);
        return this;
    }
    /**
     * Adds a {@link StepTap} step.
     * @returns The current {@link Trotsky} instance.
     */
    tap(interceptor) {
        this.append((trotsky_1.StepTap), interceptor);
        return this;
    }
    /**
     * Initializes a new {@link Trotsky} instance with the specified agent.
     * @param agent - The {@link AtpAgent} instance.
     * @returns A new {@link Trotsky} instance.
     */
    static init(agent) {
        return new Trotsky(agent);
    }
}
exports.Trotsky = Trotsky;
//# sourceMappingURL=Trotsky.js.map