import type { Did, FollowRecord, LikeRecord, PostRecord } from "@atproto/api";
import EventEmitter from "events";
/**
 * Base structure for a Jetstream message.
 */
export interface JetstreamMessageBase {
    "did": Did;
    "time_us": number;
    "kind": "commit" | "identity" | "account";
}
/**
 * Represents a commit-type Jetstream message.
 */
export interface JetstreamMessageCommit extends JetstreamMessageBase {
    "commit": {
        "rev": string;
        "operation": "create" | "update" | "delete";
        "collection": string;
        "rkey": string;
        "cid": string;
        "record"?: Partial<PostRecord> | Partial<LikeRecord> | Partial<FollowRecord>;
    };
}
/**
 * Represents an identity-type Jetstream message.
 */
export interface JetstreamMessageIdentity extends JetstreamMessageBase {
    "identity": {
        "did": Did;
        "handle": string;
        "seq": number;
        "time": string;
    };
}
/**
 * Represents an account-type Jetstream message.
 */
export interface JetstreamMessageAccount extends JetstreamMessageBase {
    "account": {
        "active": boolean;
        "did": Did;
        "seq": number;
        "time": string;
    };
}
/**
 * Union type representing all possible Jetstream message types.
 */
export type JetstreamMessage = JetstreamMessageCommit | JetstreamMessageIdentity | JetstreamMessageAccount;
/**
 * Defines the types of events emitted by the Jetstream event emitter.
 */
export type JetstreamEventTypes = {
    "error": [error: Error];
    "message": [message: JetstreamMessage];
};
/**
 * Represents a Jetstream event emitter with strongly typed events.
 */
export type JetstreamEventEmitter = EventEmitter<JetstreamEventTypes>;
/**
 * Constructs a Jetstream subscription URL with query parameters for filtering collections, DIDs, or message size.
 *
 * @param wantedCollections - The collections to subscribe to.
 * @param wantedDids - The DIDs to subscribe to.
 * @param maxMessageSizeBytes - The maximum size of messages to receive.
 * @returns The constructed subscription URL.
 */
export declare function jetstreamUrl(wantedCollections?: string[], wantedDids?: string[], maxMessageSizeBytes?: number): string;
/**
 * Builds a Jetstream event emitter to listen to messages from the Jetstream service.
 *
 * @param wantedCollections - The collections to filter messages for.
 * @param wantedDids - The DIDs to filter messages for.
 * @param maxMessageSizeBytes - The maximum size of messages to process.
 * @returns A {@link JetstreamEventEmitter} instance.
 */
export declare function buildEventEmitter(wantedCollections?: string[], wantedDids?: string[], maxMessageSizeBytes?: number): Promise<JetstreamEventEmitter>;
//# sourceMappingURL=jetstream.d.ts.map