"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.jetstreamUrl = jetstreamUrl;
exports.buildEventEmitter = buildEventEmitter;
const ws_1 = __importDefault(require("ws"));
const events_1 = __importDefault(require("events"));
const zstd_1 = require("@skhaz/zstd");
const zstd_2 = require("./zstd");
/**
 * The base URL for the Jetstream service.
 */
const JETSTREAM_URL = "wss://jetstream1.us-east.bsky.network/subscribe?compress=true";
/**
 * Constructs a Jetstream subscription URL with query parameters for filtering collections, DIDs, or message size.
 *
 * @param wantedCollections - The collections to subscribe to.
 * @param wantedDids - The DIDs to subscribe to.
 * @param maxMessageSizeBytes - The maximum size of messages to receive.
 * @returns The constructed subscription URL.
 */
function jetstreamUrl(wantedCollections = [], wantedDids = [], maxMessageSizeBytes = 0) {
    const url = new URL(JETSTREAM_URL);
    if (wantedCollections.length > 0) {
        url.searchParams.set("wantedCollections", wantedCollections.join(","));
    }
    if (wantedDids.length > 0) {
        url.searchParams.set("wantedDids", wantedDids.join(","));
    }
    if (maxMessageSizeBytes > 0) {
        url.searchParams.set("maxMessageSizeBytes", maxMessageSizeBytes.toString());
    }
    return url.toString();
}
/**
 * Builds a Jetstream event emitter to listen to messages from the Jetstream service.
 *
 * @param wantedCollections - The collections to filter messages for.
 * @param wantedDids - The DIDs to filter messages for.
 * @param maxMessageSizeBytes - The maximum size of messages to process.
 * @returns A {@link JetstreamEventEmitter} instance.
 */
async function buildEventEmitter(wantedCollections = [], wantedDids = [], maxMessageSizeBytes = 0) {
    const eventEmitter = new events_1.default();
    try {
        const url = jetstreamUrl(wantedCollections, wantedDids, maxMessageSizeBytes);
        const ws = new ws_1.default(url);
        const dict = await (0, zstd_2.downloadJetstreamZstdDict)();
        ws.on("message", async (buffer) => {
            try {
                const data = await (0, zstd_1.decompress)(buffer, { dict });
                const message = JSON.parse(data.toString());
                eventEmitter.emit("message", message);
            }
            catch {
                eventEmitter.emit("error", new Error("Failed to parse message from Jetstream"));
            }
        });
        ws.on("error", (error) => {
            eventEmitter.emit("error", error);
        });
        ws.on("close", () => {
            eventEmitter.emit("error", new Error("Jetstream connection closed"));
        });
    }
    catch {
        eventEmitter.emit("error", new Error("Failed to connect to Jetstream"));
    }
    return eventEmitter;
}
//# sourceMappingURL=jetstream.js.map