/**
 * This package provides a comprehensive suite of TypeScript classes and utilities designed
 * to facilitate interaction with the AT Protocol (ATProto) and Bluesky's social networking
 * services.
 *
 * It offers a structured approach to building and executing sequences of operations—referred
 * to as "steps"—that interact with various aspects of the protocol, such as reading user
 * profiles, posts, likes, follows, and more.
 *
 * By leveraging the schema definitions outlined in the ATProto lexicons,this package
 * ensures type safety and consistency when interfacing with ATProto's decentralized social
 * networking ecosystem.
 *
 * @remarks
 * Although all classes and methods are documented and public, the package is intended for
 * use mainly throught the {@link Trotsky} class, which serves as the main entry point using
 * a builder pattern to create and execute sequences of steps.
 *
 * @example
 * ```ts
 * import { AtpAgent } from "@atproto/api"
 * import { Trotsky } from "trotsky"
 *
 * async function main() {
 *  const agent = new AtpAgent({ service: "https://bsky.social" })
 *  await agent.login({ identifier: 'trotsky.pirhoo.com', password: 'p4ssw0rd' })
 *
 *  await Trotsky.init(agent)
 *    .searchPosts({ q: "pizza" })
 *      .take(3)
 *      .each()
 *        .reply({ text: "🍕 Pizza party! 🍕" })
 *        .wait(1000)
 *        .run()
 * }
 *
 * main()
 * ```
 *
 * @packageDocumentation
 */

import { AppBskyActorDefs } from '@atproto/api';
import type { AppBskyActorGetProfile } from '@atproto/api';
import type { AppBskyFeedDefs } from '@atproto/api';
import type { AppBskyFeedGetActorLikes } from '@atproto/api';
import type { AppBskyFeedGetAuthorFeed } from '@atproto/api';
import type { AppBskyFeedGetPosts } from '@atproto/api';
import type { AppBskyFeedPost } from '@atproto/api';
import type { AppBskyFeedSearchPosts } from '@atproto/api';
import type { AppBskyGraphDefs } from '@atproto/api';
import type { AppBskyGraphGetFollowers } from '@atproto/api';
import type { AppBskyGraphGetFollows } from '@atproto/api';
import { AppBskyGraphGetList } from '@atproto/api';
import { AppBskyGraphGetLists } from '@atproto/api';
import { AtpAgent } from '@atproto/api';
import type { AtUri } from '@atproto/api';
import type { Did } from '@atproto/api';
import EventEmitter from 'events';
import type { FollowRecord } from '@atproto/api';
import type { HeadersMap } from '@atproto/xrpc';
import type { LikeRecord } from '@atproto/api';
import { ListView } from '@atproto/api/dist/client/types/app/bsky/graph/defs';
import { PostRecord } from '@atproto/api';
import type { QueryParams } from '@atproto/api/src/client/types/app/bsky/feed/searchPosts';

/**
 * A mixin class providing reusable actor-related methods to extend step functionality.
 * These methods append specific actor-related steps to the current chain.
 *
 * @typeParam P - The parent step type.
 * @typeParam C - The context type.
 * @typeParam O - The output type.
 *
 * @public
 */
export declare abstract class ActorMixins<P, C, O> extends Step<P, C, O> {
    /**
     * Appends a step to fetch the followers of the current actor.
     *
     * @returns The appended {@link StepActorFollowers} instance.
     */
    followers(): StepActorFollowers<this>;
    /**
     * Appends a step to fetch the followings of the current actor.
     *
     * @returns The appended {@link StepActorFollowings} instance.
     */
    followings(): StepActorFollowings<this>;
    /**
     * Appends a step to fetch the liked posts of the current actor.
     *
     * @returns The appended {@link StepActorLikes} instance.
     */
    likes(): StepActorLikes<this>;
    /**
     * Appends a step to fetch the lists of the current actor.
     *
     * @returns The appended {@link StepActorLists} instance.
     */
    lists(): StepActorLists<this>;
    /**
     * Appends a step to fetch the posts of the current actor.
     *
     * @returns The appended {@link StepActorPosts} instance.
     */
    posts(): StepActorPosts<this>;
    /**
     * Appends a step to stream the posts of the current actor.
     *
     * @typeParam T - The type of the stream posts step, defaulting to {@link StepActorStreamPosts}.
     * @returns The appended {@link StepActorStreamPosts} instance.
     */
    streamPosts<T = StepActorStreamPosts<this>>(): T;
    /**
     * Appends a step to block the current actor.
     *
     * @returns The current instance for method chaining.
     */
    block(): this;
    /**
     * Appends a step to unblock the current actor.
     *
     * @returns The current instance for method chaining.
     */
    unblock(): this;
    /**
     * Appends a step to follow the current actor.
     *
     * @returns The current instance for method chaining.
     */
    follow(): this;
    /**
     * Appends a step to unfollow the current actor.
     *
     * @returns The current instance for method chaining.
     */
    unfollow(): this;
}

/**
 * Represents a Jetstream event emitter with strongly typed events.
 */
declare type JetstreamEventEmitter = EventEmitter<JetstreamEventTypes>;

/**
 * Defines the types of events emitted by the Jetstream event emitter.
 */
declare type JetstreamEventTypes = {
    "error": [error: Error];
    "message": [message: JetstreamMessage];
};

/**
 * Union type representing all possible Jetstream message types.
 */
declare type JetstreamMessage = JetstreamMessageCommit | JetstreamMessageIdentity | JetstreamMessageAccount;

/**
 * Represents an account-type Jetstream message.
 */
declare interface JetstreamMessageAccount extends JetstreamMessageBase {
    "account": {
        "active": boolean;
        "did": Did;
        "seq": number;
        "time": string;
    };
}

/**
 * Base structure for a Jetstream message.
 */
declare interface JetstreamMessageBase {
    "did": Did;
    "time_us": number;
    "kind": "commit" | "identity" | "account";
}

/**
 * Represents a commit-type Jetstream message.
 */
declare interface JetstreamMessageCommit extends JetstreamMessageBase {
    "commit": {
        "rev": string;
        "operation": "create" | "update" | "delete";
        "collection": string;
        "rkey": string;
        "cid": string;
        "record"?: Partial<PostRecord> | Partial<LikeRecord> | Partial<FollowRecord>;
    };
}

/**
 * Represents an identity-type Jetstream message.
 */
declare interface JetstreamMessageIdentity extends JetstreamMessageBase {
    "identity": {
        "did": Did;
        "handle": string;
        "seq": number;
        "time": string;
    };
}

/**
 * A mixin class providing reusable list-related methods to extend step functionality.
 * These methods append specific list-related steps to the current chain.
 *
 * @typeParam P - The parent step type.
 * @typeParam C - The context type.
 * @typeParam O - The output type.
 *
 * @public
 */
export declare abstract class ListMixins<P, C, O> extends Step<P, C, O> {
    /**
     * Appends a step to fetch the members of the current list.
     *
     * @returns The appended {@link StepListMembers} instance.
     */
    members(): StepListMembers<this, ListView, StepListMembersOutput>;
}

/**
 * A mixin class providing reusable post-related methods to extend step functionality.
 * These methods append specific post-related steps to the current chain.
 *
 * @typeParam P - The parent type of this step.
 * @typeParam C - The context type.
 * @typeParam O - The output type.
 * @public
 */
export declare abstract class PostMixins<P, C, O> extends Step<P, C, O> {
    /**
     * Appends a step to reply to the current post.
     *
     * @param record - The parameters for the reply, which may be resolved at runtime.
     * @returns The appended {@link StepPostReply} instance.
     */
    reply(record: ResolvablePostReplyParams): StepPostReply<this>;
    /**
     * Appends a step to like the current post.
     *
     * @returns The appended {@link StepPostLike} instance.
     */
    like(): StepPostLike<this>;
    /**
     * Appends a step to repost the current post.
     *
     * @returns The appended {@link StepPostRepost} instance.
     */
    repost(): StepPostRepost<this>;
    /**
     * Appends a step to retrieve the author of the current post.
     *
     * @returns The appended {@link StepPostAuthor} instance.
     */
    author(): StepPostAuthor<this>;
}

/**
 * Type representing the parameters for a post reply, including text and optional additional metadata.
 * @public
 */
export declare type PostReplyParams = Partial<AppBskyFeedPost.Record> & {
    "text": string;
};

/**
 * Represents a value that can be resolved dynamically. It may be:
 * - A direct value of type `T`
 * - A promise resolving to type `T`
 * - A function returning a value of type `T` or a promise resolving to `T`, with a {@link Step} as its argument.
 *
 * @typeParam T - The type of the resolvable value.
 */
declare type Resolvable<T> = T | Promise<T> | ((step: Step) => T | Promise<T>);

/**
 * Type representing a value that can be resolved into {@link PostReplyParams}.
 * @public
 */
export declare type ResolvablePostReplyParams = Resolvable<PostReplyParams>;

/**
 * Type representing a {@link StepCreatePostParams} type
 * that may be resolved dynamically at runtime.
 * @public
 */
export declare type ResolvableStepCreatePostParams = Resolvable<StepCreatePostParams>;

/**
 * Represents an abstract step in a sequence managed by the {@link Trotsky} framework.
 * @typeParam P - Type of the parent object.
 * @typeParam C - Type of the context object.
 * @typeParam O - Type of the output object.
 * @public
 */
export declare abstract class Step<P = StepBuilder, C = unknown, O = unknown> extends StepBuilder {
    /** @internal */
    _parent: P;
    /** @internal */
    _context: C | null;
    /** @internal */
    _output: O | null;
    constructor(agent: AtpAgent, parent: P);
    /**
     * Sets the parent.
     * @param context - The parent object.
     * @returns The current {@link Trotsky} instance.
     */
    withParent(parent: StepBuilder): this;
    /**
     * Sets the context.
     * @param context - The context object.
     * @returns The current {@link Trotsky} instance.
     */
    withContext(context: C): this;
    /**
     * Sets the output.
     * @param output - The output object.
     * @returns The current {@link Trotsky} instance.
     */
    withOutput(output: O): this;
    /**
     * Adds a {@link StepWait} step.
     * @param duration - The duration to wait, in milliseconds.
     * @returns The current {@link Step} instance.
     */
    wait(duration?: number): this;
    /**
     *Adds a {@link StepWhen} step.
     * @param predicate - The predicate function.
     * @returns The current {@link Step} instance.
     */
    when(predicate: Resolvable<StepWhenPredicate>): this;
    /**
     * Adds a {@link StepTap} step.
     * @returns The current {@link Step} instance.
     */
    tap(interceptor: StepTapInterceptor): this;
    /**
     * Adds a {@link StepSave} step.
     * @param path - The path of the JSON file to save the output. If not provided, the file path will be created using the current step name.
     * @returns The current {@link Step} instance.
     */
    save(path?: StepSavePath): this;
    /**
     * Retreive the configuration value for a given key on the current
     * step or inherite the value from the parent step.
     * @param key - The configuration key.
     * @returns The configuration value.
     */
    config(key: string): unknown;
    /**
     * Updates the configuration value for a given key on the current step.
     * @param key - The configuration key.
     * @param value - The new value.
     * @returns The current {@link Step} instance.
     */
    config(key: string, value: unknown): this;
    /**
     * Updates the configuration object on the current step.
     * @param config - The new configuration object.
     * @returns The current {@link Step} instance.
     */
    config(config: StepBuilderConfig): this;
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the constructor.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Retrieves the parent object of this instance.
     * @returns The parent object.
     */
    back(): P;
    /**
     * Ends the sequence of steps.
     * @returns The top-level {@link Trotsky} instance.
     */
    end(): StepBuilder;
    /**
     * Abstract method that must be implemented by subclasses to define the behavior of the step.
     * @returns A promise that resolves when the step is applied.
     */
    abstract apply(): Promise<void>;
    /**
     * Runs all steps in the sequence and returns the current {@link Step} instance.
     */
    runHere(): Promise<this>;
    /**
     * Checks if this instance is part of a {@link StepBuilderList}.
     */
    get isStepBuilderListEntry(): boolean;
    get isStepWhen(): boolean;
    /**
     * Retrieves or sets the current context.
     */
    get context(): C;
    set context(value: C);
    /**
     * Retrieves or sets the current output.
     */
    get output(): O | null;
    set output(value: O | null);
}

/**
 * Represents a step for retrieving and handling an actor's profile using the Bluesky API.
 * @typeParam P - Type of the parent object.
 * @typeParam C - Type of the context object.
 * @typeParam O - Type of the output object, extending {@link StepActorOutput}.
 * @public
 */
export declare class StepActor<P = StepBuilder, C = null, O extends StepActorOutput = StepActorOutput> extends ActorMixins<P, C, O> {
    /** @internal */
    _param: Resolvable<StepActorParam>;
    /**
     * Initializes a new {@link StepActor} instance.
     * @param agent - The {@link AtpAgent} instance for API interactions.
     * @param parent - The parent object associated with this step.
     * @param param - The parameter representing the actor to retrieve, which can be a resolvable value.
     */
    constructor(agent: AtpAgent, parent: P, param: Resolvable<StepActorParam>);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Applies the step by resolving the actor parameter and fetching the actor's profile.
     * Sets the profile data as the output of this step.
     * @returns A promise that resolves when the step is complete.
     */
    apply(): Promise<void>;
}

/**
 * Represents a step that performs an actor block operation using the Bluesky API.
 * @typeParam P - Type of the parent step, extending {@link StepActor}.
 * @typeParam C - Type of the context object, extending {@link StepActorOutput}.
 * @typeParam O - Type of the output object.
 * @public
 */
export declare class StepActorBlock<P = StepActor, C extends StepActorOutput = StepActorOutput, O = null> extends Step<P, C, O> {
    /**
     * Applies the step by performing a block operation.
     * Requires the context to provide the `did` of the actor to block.
     * @throws Error
     * if no context is found.
     */
    apply(): Promise<void>;
}

/**
 * Represents a step that performs an actor follow operation using the agent.
 * @typeParam P - Type of the parent step, extending {@link StepActor}.
 * @typeParam C - Type of the context object, extending {@link StepActorOutput}.
 * @typeParam O - Type of the output object.
 * @public
 */
export declare class StepActorFollow<P = StepActor, C extends StepActorOutput = StepActorOutput, O = null> extends Step<P, C, O> {
    /**
     * Applies the step by performing a follow operation.
     * Requires the context to provide the `did` of the actor to follow.
     * @throws
     * Error if no context is found.
     */
    apply(): Promise<void>;
}

/**
 * Represents a step for retrieving an actor's followers using the Bluesky API.
 * Supports paginated retrieval of followers.
 * @typeParam P - Type of the parent step, extending {@link StepActor}.
 * @typeParam C - Type of the context object, extending {@link StepActorOutput}.
 * @typeParam O - Type of the output object, extending {@link StepActorFollowersOutput}.
 * @public
 */
export declare class StepActorFollowers<P = StepActor, C extends StepActorOutput = StepActorOutput, O extends StepActorFollowersOutput = StepActorFollowersOutput> extends StepActors<P, C, O> {
    /**
     * Applies pagination to retrieve followers and sets the output.
     * Fetches paginated results using the agent and appends them to the output.
     */
    applyPagination(): Promise<void>;
    /**
     * Generates query parameters for retrieving followers, including the optional cursor.
     * @param cursor - The cursor for paginated queries.
     * @returns The query parameters for retrieving followers.
     * @throws
     * Error if no context is found.
     */
    private queryParams;
}

/**
 * Type representing the output of the followers retrieved by {@link StepActorFollowers}.
 * @public
 */
export declare type StepActorFollowersOutput = AppBskyGraphGetFollowers.OutputSchema["followers"];

/**
 * Type representing the query parameters for retrieving followers.
 * @public
 */
export declare type StepActorFollowersQueryParams = AppBskyGraphGetFollowers.QueryParams;

/**
 * Type representing the cursor for paginated queries.
 * @public
 */
export declare type StepActorFollowersQueryParamsCursor = StepActorFollowersQueryParams["cursor"] | undefined;

/**
 * Represents a step for retrieving an actor's followings using the Bluesky API.
 * Supports paginated retrieval of followings.
 * @typeParam P - Type of the parent step, extending {@link StepActor}.
 * @typeParam C - Type of the context object, extending {@link StepActorOutput}.
 * @typeParam O - Type of the output object, extending {@link StepActorFollowingsOutput}.
 * @public
 */
export declare class StepActorFollowings<P = StepActor, C extends StepActorOutput = StepActorOutput, O extends StepActorFollowingsOutput = StepActorFollowingsOutput> extends StepActors<P, C, O> {
    /**
     * Applies pagination to retrieve followings and sets the output.
     * Fetches paginated results using the agent and appends them to the output.
     */
    applyPagination(): Promise<void>;
    /**
     * Generates query parameters for retrieving followings, including the optional cursor.
     * @param cursor - The cursor for paginated queries.
     * @returns The query parameters for retrieving followings.
     * @throws
     * Error if no context is found.
     */
    queryParams(cursor: StepActorFollowingsQueryParamsCursor): StepActorFollowingsQueryParams;
}

/**
 * Type representing the output of the followings retrieved by {@link StepActorFollowings}.
 * @public
 */
export declare type StepActorFollowingsOutput = AppBskyGraphGetFollows.OutputSchema["follows"];

/**
 * Type representing the query parameters for retrieving followings.
 * @public
 */
export declare type StepActorFollowingsQueryParams = AppBskyGraphGetFollows.QueryParams;

/**
 * Type representing the cursor for paginated queries.
 * @public
 */
export declare type StepActorFollowingsQueryParamsCursor = StepActorFollowingsQueryParams["cursor"] | undefined;

/**
 * Represents a step for retrieving an actor's likes using the Bluesky API.
 * Supports paginated retrieval of likes.
 * @typeParam P - Type of the parent step, extending {@link StepActor}.
 * @typeParam C - Type of the context object, extending {@link StepActorOutput}.
 * @typeParam O - Type of the output object, extending {@link StepActorLikesOutput}.
 * @public
 */
export declare class StepActorLikes<P = StepActor, C extends StepActorOutput = StepActorOutput, O extends StepActorLikesOutput = StepActorLikesOutput> extends StepPosts<P, C, O> {
    /**
     * Applies pagination to retrieve likes and sets the output.
     * Fetches paginated results using the agent and appends them to the output.
     */
    applyPagination(): Promise<void>;
    /**
     * Generates query parameters for retrieving likes, including the optional cursor.
     * @param cursor - The cursor for paginated queries.
     * @returns The query parameters for retrieving likes.
     * @throws
     * Error if no context is found.
     */
    queryParams(cursor: StepActorLikesQueryParamsCursor): StepActorLikesQueryParams;
}

/**
 * Type representing the output of the likes retrieved by {@link StepActorLikes}.
 * @public
 */
export declare type StepActorLikesOutput = StepPostsOutput;

/**
 * Type representing the query parameters for retrieving actor likes.
 * @public
 */
export declare type StepActorLikesQueryParams = AppBskyFeedGetActorLikes.QueryParams;

/**
 * Type representing the cursor for paginated queries.
 * @public
 */
export declare type StepActorLikesQueryParamsCursor = StepActorLikesQueryParams["cursor"] | undefined;

/**
 * Represents step for retrieving the lists associated with a specific actor, extending the {@link StepLists} functionality.
 *
 * @typeParam P - The parent step type, defaulting to {@link StepActor}.
 * @typeParam C - The context type, defaulting to {@link StepActorOutput}.
 * @typeParam O - The output type, defaulting to {@link StepActorListsOutput}.
 *
 * @public
 */
export declare class StepActorLists<P = StepActor, C extends StepActorOutput = StepActorOutput, O extends StepListsOutput = StepActorListsOutput> extends StepLists<P, C, O> {
    /**
     * Applies pagination to retrieve all lists associated with the actor, handling multiple pages of results if necessary.
     *
     * @returns A promise that resolves when pagination is complete.
     */
    applyPagination(): Promise<void>;
    /**
     * Generates query parameters for retrieving an actor's lists, including the optional cursor.
     *
     * @param cursor - The cursor for paginated queries.
     * @returns The query parameters for retrieving the actor's lists.
     * @throws Error if no context is found.
     */
    queryParams(cursor: StepActorListsQueryParamsCursor): StepActorListsQueryParams;
}

/**
 * Type representing the output type for the {@link StepActorLists} step, which is an array
 * of lists associated with an actor.
 *
 * @public
 */
export declare type StepActorListsOutput = StepListsOutput;

/**
 * Type representing the query parameters for retrieving an actor's lists.
 *
 * @public
 */
export declare type StepActorListsQueryParams = AppBskyGraphGetLists.QueryParams;

/**
 * Type representing the cursor type for paginated queries when retrieving an actor's lists.
 *
 * @public
 */
export declare type StepActorListsQueryParamsCursor = StepActorListsQueryParams["cursor"] | undefined;

/**
 * Type for the output of the {@link StepActor} class.
 * Matches the schema of the actor profile from the Bluesky API.
 * @public
 */
export declare type StepActorOutput = AppBskyActorGetProfile.OutputSchema;

/**
 * Type for the parameter passed to the {@link StepActor} class.
 * Represents the identifier of an actor (e.g., handle or DID).
 * @public
 */
export declare type StepActorParam = string;

/**
 * Represents a step for retrieving an actor's posts using the Bluesky API.
 * Supports paginated retrieval of posts.
 * @typeParam P - Type of the parent step, extending {@link StepActor}.
 * @typeParam C - Type of the context object, extending {@link StepActorOutput}.
 * @typeParam O - Type of the output object, extending {@link StepActorPostsOutput}.
 * @public
 */
export declare class StepActorPosts<P = StepActor, C extends StepActorOutput = StepActorOutput, O extends StepActorPostsOutput = StepActorPostsOutput> extends StepPosts<P, C, O> {
    /**
     * Applies pagination to retrieve posts and sets the output.
     * Fetches paginated results using the agent and appends them to the output.
     */
    applyPagination(): Promise<void>;
    /**
     * Generates query parameters for retrieving posts, including the optional cursor.
     * @param cursor - The cursor for paginated queries.
     * @returns The query parameters for retrieving posts.
     * @throws
     * Error if no context is found.
     */
    queryParams(cursor: StepActorPostsQueryParamsCursor): StepActorPostsQueryParams;
}

/**
 * Type representing the output of the posts retrieved by {@link StepActorPosts}.
 * @public
 */
export declare type StepActorPostsOutput = StepPostsOutput;

/**
 * Type representing the query parameters for retrieving actor posts.
 * @public
 */
export declare type StepActorPostsQueryParams = AppBskyFeedGetAuthorFeed.QueryParams;

/**
 * Type representing the cursor for paginated queries.
 * @public
 */
export declare type StepActorPostsQueryParamsCursor = StepActorPostsQueryParams["cursor"] | undefined;

/**
 * An abstract class that extends {@link StepBuilderList} to handle a list of actor profiles.
 * It defines a collection of steps dealing with actor entries (`StepActorsEntry`),
 * which can be appended via the `each()` method.
 *
 * @typeParam P - The parent step builder type, defaults to `StepBuilder`.
 * @typeParam C - The context type, defaults to `null`.
 * @typeParam O - The output type, extending {@link StepActorsOutput}, defaults to `StepActorsOutput`.
 * @public
 */
export declare class StepActors<P = StepBuilder, C = null, O extends StepActorsOutput = StepActorsOutput> extends StepBuilderList<P, C, O> {
    /**
     * Stores the list of appended actor entries.
     * @internal
     */
    _steps: StepActorsEntry<this, null, null>[];
    /**
     * The parameter passed to the {@link StepActors} class to retreive the actors
     * @internal
     */
    _param: Resolvable<StepActorsParam>;
    /**
     * Initializes a new {@link StepActors} instance.
     * @param agent - The {@link AtpAgent} instance for API interactions.
     * @param parent - The parent object associated with this step.
     * @param param - The parameter representing the actors to retrieve, which can be a resolvable value.
     */
    constructor(agent: AtpAgent, parent: P, param: Resolvable<StepActorsParam>);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Appends a new {@link StepActorsEntry} instance to the step list
     * and returns the newly created entry.
     *
     * @returns A new step entry configured for actor processing.
     */
    each(iterator?: StepBuilderListIterator): StepActorsEntry<this>;
    /**
     * Applies the step by resolving the actor parameter and fetching the actors' profiles.
     * Sets the profile data as the output of this step.
     * @returns A promise that resolves when the step is complete.
     */
    applyPagination(): Promise<void>;
}

/**
 * Represents a single entry step for processing actor information.
 * Extends {@link ActorMixins} to provide common actor-related methods.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepActors}.
 * @typeParam C - The child type for this step, defaulting to `unknown`.
 * @typeParam O - The output type, defaulting to `unknown`.
 */
export declare class StepActorsEntry<P = StepActors, C = unknown, O = unknown> extends ActorMixins<P, C, O> {
    /**
     * Applies the step's logic but do nothing by default. This method is
     * usually be overridden by child classes but will not throw an error if not.
     * @override
     */
    apply(): Promise<void>;
}

/**
 * Type representing the output schema for a list of actors' profiles.
 * @public
 */
export declare type StepActorsOutput = Partial<AppBskyActorDefs.ProfileView>[];

/**
 * Type for the parameter passed to the {@link StepActors} class.
 * Represents the identifiers of several actors (e.g., handle or DID).
 * Bluesky currently limit this to 25 actors.
 * @public
 */
export declare type StepActorsParam = string[];

/**
 * @experimental
 *
 * A specialized step that extends {@link StepStreamPosts} to stream an actor's posts.
 * It uses a Jetstream-based event emitter filtered by the actor's DID.
 *
 * @typeParam P - The parent type of this step.
 * @typeParam C - The context or child output type, extending {@link StepActorOutput}.
 * @typeParam O - The final output type of this step, defaulting to {@link StepActorStreamPostsOutput}.
 */
export declare class StepActorStreamPosts<P, C extends StepActorOutput = StepActorOutput, O = StepActorStreamPostsOutput> extends StepStreamPosts<P, C, O> {
    /**
     * Provides an event emitter that listens for "app.bsky.feed.post" events associated with this actor.
     * Throws an error if no context (i.e., actor DID) is available.
     *
     * @returns A promise resolving to the Jetstream event emitter.
     */
    buildEventEmitter(): Promise<JetstreamEventEmitter>;
}

/**
 * Combines a Jetstream message commit with a partial post record.
 * Typically represents the streamed output of a single post event.
 * @public
 */
export declare type StepActorStreamPostsOutput = JetstreamMessageCommit & {
    "record": Partial<PostRecord>;
};

/**
 * Represents step that unblocks the current actor (if blocked).
 *
 * @typeParam P - The parent step type, defaulting to {@link StepActor}.
 * @typeParam C - The context or child output type, defaulting to {@link StepActorOutput}.
 * @typeParam O - The output type produced by this step, defaulting to `null`.
 * @public
 */
export declare class StepActorUnblock<P = StepActor, C extends StepActorOutput = StepActorOutput, O = null> extends Step<P, C, O> {
    /**
     * Applies the unblock logic:
     * 1. Retrieves the current block list.
     * 2. Checks if the actor is actually blocked.
     * 3. If blocked, deletes the block record to unblock the actor.
     *
     * @throws Error
     * If no context is available for the step, indicating the actor is unknown.
     */
    apply(): Promise<void>;
}

/**
 * Represents step that unfollows the current actor (if followed).
 *
 * @typeParam P - The parent step type, defaulting to {@link StepActor}.
 * @typeParam C - The context or child output type, defaulting to {@link StepActorOutput}.
 * @typeParam O - The output type produced by this step, defaulting to `null`.
 * @public
 */
export declare class StepActorUnfollow<P = StepActor, C extends StepActorOutput = StepActorOutput, O = null> extends Step<P, C, O> {
    /**
     * Applies the unfollow logic:
     * 1. Retrieves the current actor's relationship with the specified context actor.
     * 2. Checks if the current actor is indeed following the specified context actor.
     * 3. If so, calls the API to remove the follow relationship.
     *
     * @throws Error
     * If no context is available for the step, indicating the actor is unknown.
     */
    apply(): Promise<void>;
}

/**
 * Represents a builder for a sequence of steps. This class is used internally by the {@link Trotsky} framework.
 */
export declare abstract class StepBuilder {
    /** @internal */
    _agent: AtpAgent;
    /** @internal */
    _steps: Step<this>[];
    /** @internal */
    _config: StepBuilderConfig;
    /**
     * Initializes a new {@link StepBuilder} instance.
     * @param agent - The {@link AtpAgent} instance for API interactions.
     */
    constructor(agent: AtpAgent);
    /**
     * Updates the agent instance.
     * @param agent - The new {@link AtpAgent} instance.
     *
     * @returns The current {@link Trotsky} instance.
     */
    withAgent(agent: AtpAgent): this;
    /**
     * Retreive the configuration value for a given key.
     * @param key - The configuration key.
     * @returns The configuration value.
     */
    config(key: string): unknown;
    /**
     * Updates the configuration value for a given key.
     * @param key - The configuration key.
     * @param value - The new value.
     * @returns The current {@link StepBuilder} instance.
     */
    config(key: string, value: unknown): this;
    /**
     * Updates the configuration object.
     * @param config - The new configuration object.
     * @returns The current {@link StepBuilder} instance.
     */
    config(config: StepBuilderConfig): this;
    /**
     * Clones the current instance, including all steps and configuration.
     *
     * @param rest - Additional arguments to pass to the constructor.
     * @returns A new instance of the current class.
     */
    clone(...rest: unknown[]): this;
    /**
     * Clears all steps from the sequence.
     * @returns The current {@link StepBuilder} instance.
     */
    clear(): this;
    /**
     * Appends a new step to the sequence.
     * @param type - The step class.
     * @param args - Arguments for the step.
     * @returns The new step instance.
     */
    append<Type extends Step<this>>(type: new (agent: AtpAgent, parent: this, ...args: any[]) => Type, ...args: unknown[]): Type;
    /**
     * Adds a step to the sequence.
     * @param stepOrIndex - The step instance or index to remove.
     * @returns The current {@link StepBuilder} instance.
     */
    slice(stepOrIndex: Step<this> | number): this;
    /**
     * Pushes a step to the sequence.
     * @param step - The step instance.
     * @returns The current {@link StepBuilder} instance.
     */
    push(...steps: Step<this>[]): this;
    /**
     * Runs the sequence of steps, returning the top-level {@link StepBuilder} instance.
     */
    run(): any;
    /**
     * Runs all steps in the sequence.
     * @returns The current {@link Trotsky} instance.
     */
    runAll(): Promise<this>;
    /**
     * Schedules the sequence to run at a given interval.
     * @param expression - The cron {@link https://github.com/kelektiv/node-cron | expression} defining the schedule.
     * @returns A cron job instance.
     */
    schedule(expression: string): any;
    /**
     * Applies all steps in the sequence.
     */
    applyAll(): Promise<void>;
    /**
     * Checks if this is a top-level {@link Trotsky} instance.
     */
    get isTrotsky(): boolean;
    /**
     * Retrieves all steps in the sequence.
     */
    get steps(): Step[];
    /**
     * Retrieves a flattened list of all steps, including nested ones.
     */
    get flattenSteps(): Step[];
    /**
     * Retrieves the current {@link AtpAgent} instance.
     */
    get agent(): AtpAgent;
}

/**
 * Represents the configuration object for a {@link StepBuilder} instance.
 * @public
 */
export declare type StepBuilderConfig = object;

/**
 * Abstract class representing a step that processes paginated lists.
 *
 * @typeParam P - The parent type of the step, defaulting to {@link StepBuilder}.
 * @typeParam C - The child context type, defaulting to `unknown`.
 * @typeParam O - The output type of the step, defaulting to {@link StepBuilderListOutput}.
 * @public
 */
export declare abstract class StepBuilderList<P = StepBuilder, C = unknown, O extends StepBuilderListOutput = StepBuilderListOutput> extends Step<P, C, O> {
    /**
     * Holds the list of steps to be executed for each entry in the list.
     */
    _steps: StepBuilderListEntry<this>[];
    /**
     * Number of items to take from the list. Defaults to `Infinity`.
     */
    _take: Resolvable<number>;
    /**
     * Number of items to skip from the start of the list. Defaults to `0`.
     */
    _skip: Resolvable<number>;
    /**
     * Optional tterator function to be executed for each item in the list.
     */
    _iterator?: StepBuilderListIterator;
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Sets the number of items to take from the list.
     *
     * @param take - The number of items to take, resolved at runtime if necessary.
     * @returns The current instance for method chaining.
     */
    take(take: Resolvable<number>): this;
    /**
     * Sets the number of items to skip from the start of the list.
     *
     * @param skip - The number of items to skip, resolved at runtime if necessary.
     * @returns The current instance for method chaining.
     */
    skip(skip: Resolvable<number>): this;
    /**
     * Sets the iterator function to be executed for each item in the list.
     *
     * @param iterator - The iterator function to be executed.
     * @returns The current instance for method chaining.
     */
    withIterator(iterator?: StepBuilderListIterator): this;
    /**
     * Appends a new entry processing step to the list.
     *
     * @param iterator - Optional iterator function to be executed for each item in the list.
     * @returns The appended {@link StepBuilderListEntry} instance.
     */
    each(iterator?: StepBuilderListIterator): StepBuilderListEntry<this, unknown, unknown>;
    /**
     * Applies the list processing logic, including pagination and execution of child steps for each item in the output.
     *
     * @throws Error
     * If `applyPagination` is not implemented by a child class.
     */
    apply(): Promise<void>;
    /**
     * Abstract method to handle pagination. Must be implemented by subclasses.
     */
    abstract applyPagination(): Promise<void>;
    /**
     * Handles pagination logic for retrieving records, fetching items in batches until the desired size is reached.
     *
     * @typeParam T - The type of the final output array.
     * @typeParam R - The response type of the API call.
     *
     * @param attribute - The attribute containing the paginated records.
     * @param fn - The function to fetch paginated data, taking the current cursor as input.
     * @returns A promise that resolves to an array of items after applying skip and take logic.
     */
    paginate<T, R extends StepBuilderListResponse>(attribute: string, fn: (cursor: StepBuilderListOutputSchemaCursor) => Promise<R>): Promise<T>;
}

/**
 * Represents an individual entry step within a {@link StepBuilderList}.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepBuilderList}.
 * @typeParam C - The child context type, defaulting to `unknown`.
 * @typeParam O - The output type, defaulting to `unknown`.
 * @public
 */
export declare class StepBuilderListEntry<P = StepBuilderList, C = unknown, O = unknown> extends Step<P, C, O> {
    /**
     * Applies the step's logic but do nothing by default. This method is
     * usually be overridden by child classes but will not throw an error if not.
     * @override
     */
    apply(): Promise<void>;
}

/**
 * Iterator function to be executed for each item in the list.
 * @public
 */
export declare type StepBuilderListIterator = null | ((step: StepBuilderListEntry) => Promise<void> | void);

/**
 * Represents the output of a list step as an array of items.
 * @public
 */
export declare type StepBuilderListOutput = unknown[];

/**
 * Interface defining the schema for a paginated list output.
 * @public
 */
declare interface StepBuilderListOutputSchema {
    "cursor"?: StepBuilderListOutputSchemaCursor;
    "hitsTotal"?: number;
    [k: string]: unknown;
}

/**
 * Represents the cursor for pagination in a list schema.
 * @public
 */
declare type StepBuilderListOutputSchemaCursor = string | undefined;

/**
 * Interface representing the structure of a paginated response.
 * @public
 */
declare interface StepBuilderListResponse {
    "success": boolean;
    "headers": HeadersMap;
    "data": StepBuilderListOutputSchema;
}

/**
 * @experimental
 *
 * Abstract class representing a step for streaming messages using a Jetstream event emitter.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepBuilder}.
 * @typeParam C - The context type, defaulting to `unknown`.
 * @typeParam O - The output type, defaulting to {@link JetstreamMessage}.
 * @public
 */
export declare abstract class StepBuilderStream<P = StepBuilder, C = unknown, O = JetstreamMessage> extends Step<P, C, O> {
    /**
     * Holds the list of steps to be executed for each streamed entry.
     */
    _steps: StepBuilderStreamEntry<this>[];
    /**
     * Initializes the StepBuilderStream instance with the given agent and parent step.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     */
    constructor(agent: AtpAgent, parent: P);
    /**
     * Appends a new stream entry step to the current list and returns it.
     *
     * @returns The newly appended {@link StepBuilderStreamEntry} instance.
     */
    each(): StepBuilderStreamEntry<this, unknown, unknown>;
    /**
     * Build the Jetstream event emitter. Must be implemented by subclasses.
     *
     * @returns A promise resolving to the Jetstream event emitter.
     */
    abstract buildEventEmitter(): Promise<JetstreamEventEmitter>;
    /**
     * Resolves the output of the current step from the given message.
     *
     * @param message - The incoming Jetstream message.
     * @returns A promise resolving to the step's output.
     */
    resolveOutput(message: JetstreamMessage): Promise<O>;
    /**
     * Applies the stream logic by listening to events from the event emitter and processing them sequentially.
     * Messages are handled using a queue to ensure sequential execution.
     *
     * @override
     * @returns A promise that resolves when the stream processing is complete or rejects on error.
     */
    apply(): Promise<void>;
    /**
     * Processes a Jetstream message by executing child steps for valid commit events.
     *
     * @param message - The incoming Jetstream message.
     * @returns A promise that resolves when the message has been processed.
     */
    onMessage(message: JetstreamMessage): Promise<void>;
}

/**
 * @experimental
 *
 * Represents an individual entry step within a {@link StepBuilderStream}.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepBuilderStream}.
 * @typeParam C - The context type, defaulting to `unknown`.
 * @typeParam O - The output type, defaulting to `unknown`.
 * @public
 */
export declare class StepBuilderStreamEntry<P = StepBuilderStream, C = unknown, O = unknown> extends Step<P, C, O> {
    /**
     * Applies the step's logic but do nothing by default. This method is
     * usually be overridden by child classes but will not throw an error if not.
     * @override
     */
    apply(): Promise<void>;
}

/**
 * Represents step that creates a new post on Bluesky using the specified record parameters.
 *
 * @typeParam P - The parent step type, defaulting to {@link StepBuilder}.
 * @typeParam C - The child context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to {@link StepCreatePostOutput}.
 * @public
 */
export declare class StepCreatePost<P = StepBuilder, C = null, O extends StepCreatePostOutput = StepCreatePostOutput> extends Step<P, C, O> {
    /**
     * Stores the parameters needed to create a post, which may be resolved at runtime.
     */
    _record: ResolvableStepCreatePostParams;
    /**
     * Initializes the create-post step with the provided agent, parent, and record parameters.
     * @param agent - The AT protocol agent for making API calls.
     * @param parent - The parent step in the chain.
     * @param record - The post record parameters, possibly resolvable at runtime.
     */
    constructor(agent: AtpAgent, parent: P, record: ResolvableStepCreatePostParams);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Executes the create-post step by sending a request to the Bluesky service
     * and assigning the returned URI and CID to `this.output`.
     */
    apply(): Promise<void>;
    /**
     * Resolves the post creation parameters from a potentially dynamic input source.
     * @returns A promise that resolves to the fully populated {@link StepCreatePostParams}.
     */
    queryParams(): Promise<StepCreatePostParams>;
}

/**
 * Defines the output of a create-post operation,
 * containing the post's unique URI and CID.
 * @public
 */
export declare type StepCreatePostOutput = {
    "uri": string;
    "cid": string;
};

/**
 * Defines the parameters for creating a new post.
 * Consists of a partial {@link AppBskyFeedPost.Record} but excludes `createdAt`.
 * @public
 */
export declare type StepCreatePostParams = Partial<AppBskyFeedPost.Record> & Omit<AppBskyFeedPost.Record, "createdAt">;

/**
 * Represents a step for retrieving a list by its URI.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepBuilder}.
 * @typeParam C - The child context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to {@link StepListOutput}.
 * @public
 */
export declare class StepList<P = StepBuilder, C = null, O extends StepListOutput = StepListOutput> extends ListMixins<P, C, O> {
    /**
     * The URI of the list to retrieve, which can be resolved dynamically at runtime.
     * @internal
     */
    _uri: Resolvable<StepListUri>;
    /**
     * Initializes the StepList instance with the given agent, parent, and URI.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param uri - The URI of the list to retrieve, possibly resolvable at runtime.
     */
    constructor(agent: AtpAgent, parent: P, uri: Resolvable<StepListUri>);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Applies the step logic to retrieve the list and sets the output to the retrieved list's data.
     * If no lists are found, the output is set to `null`.
     *
     * @override
     */
    apply(): Promise<void>;
    /**
     * Resolves the query parameters for retrieving the list, including its URI.
     *
     * @returns A promise that resolves to the query parameters.
     */
    queryParams(): Promise<StepListQueryParams>;
}

/**
 * Represents step for retrieving the members of a specific list, extending the {@link StepActors} functionality.
 *
 * @typeParam P - The parent step type, defaulting to {@link StepList}.
 * @typeParam C - The context type, defaulting to {@link StepListOutput}.
 * @typeParam O - The output type, defaulting to {@link StepListMembersOutput}.
 *
 * @public
 */
export declare class StepListMembers<P = StepList, C extends StepListOutput = StepListOutput, O extends StepListMembersOutput = StepListMembersOutput> extends StepActors<P, C, O> {
    /**
     * Applies pagination to retrieve all members of the list, handling multiple pages of results if necessary.
     *
     * @returns A promise that resolves when pagination is complete.
     */
    applyPagination(): Promise<void>;
    /**
     * Generates query parameters for retrieving a list and its members, including the optional cursor.
     *
     * @param cursor - The cursor for paginated queries.
     * @returns The query parameters for retrieving the list.
     * @throws Error if no context is found.
     */
    private queryParams;
}

/**
 * Represents the output type for the {@link StepListMembers} step, which is an array of actor profiles.
 * @public
 */
export declare type StepListMembersOutput = AppBskyActorDefs.ProfileView[];

/**
 * Represents the query parameters for retrieving a list's members.
 * @public
 */
export declare type StepListMembersQueryParams = AppBskyGraphGetList.QueryParams;

/**
 * Represents the cursor type for paginated queries when retrieving list members.
 * @public
 */
export declare type StepListMembersQueryParamsCursor = StepListMembersQueryParams["cursor"] | undefined;

/**
 * Represents the output of a retrieved list, including its URI and CID
 */
export declare type StepListOutput = AppBskyGraphDefs.ListView;

/**
 * Defines the query parameters for retrieving a list.
 * @public
 */
export declare type StepListQueryParams = AppBskyGraphGetList.QueryParams;

/**
 * Abstract class representing a list of Lists to process.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepBuilder}.
 * @typeParam C - The child context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to {@link StepListsOutput}.
 * @public
 */
export declare abstract class StepLists<P = StepBuilder, C = null, O extends StepListsOutput = StepListsOutput> extends StepBuilderList<P, C, O> {
    /**
     * Holds the list of steps to be executed for each List entry.
     */
    _steps: StepListsEntry<this>[];
    /**
     * Appends a new List entry step to the current list and returns it.
     *
     * @param iterator - Optional iterator function to be executed for each item in the list.
     * @returns The newly appended {@link StepListsEntry} instance.
     */
    each(iterator?: StepBuilderListIterator): StepListsEntry<this>;
}

/**
 * Represents an individual entry step within a {@link StepLists} list.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepLists}.
 * @typeParam C - The context type, defaulting to {@link StepListsOutput}.
 * @typeParam O - The output type, defaulting to `unknown`.
 * @public
 */
export declare class StepListsEntry<P = StepLists, C extends StepListsOutput = StepListsOutput, O = unknown> extends ListMixins<P, C, O> {
    /**
     * Applies the step's logic but do nothing by default. This method is
     * usually be overridden by child classes but will not throw an error if not.
     * @override
     */
    apply(): Promise<void>;
}

/**
 * Represents the output of a Lists step, consisting of an array of List views.
 * @public
 */
export declare type StepListsOutput = AppBskyGraphGetLists.OutputSchema["lists"];

/**
 * Represents a list's URI, which can be a string or an {@link AtUri}.
 * @public
 */
export declare type StepListUri = string | AtUri;

/**
 * Represents a step for retrieving a post by its URI.
 * Extends {@link PostMixins} for post-related operations.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepBuilder}.
 * @typeParam C - The child context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to {@link StepPostOutput}.
 * @public
 */
export declare class StepPost<P = StepBuilder, C = null, O extends StepPostOutput = StepPostOutput> extends PostMixins<P, C, O> {
    /**
     * The URI of the post to retrieve, which can be resolved dynamically at runtime.
     */
    _uri: Resolvable<StepPostUri>;
    /**
     * Initializes the StepPost instance with the given agent, parent, and URI.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param uri - The URI of the post to retrieve, possibly resolvable at runtime.
     */
    constructor(agent: AtpAgent, parent: P, uri: Resolvable<StepPostUri>);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Applies the step logic to retrieve the post and sets the output to the retrieved post's data.
     * If no posts are found, the output is set to `null`.
     *
     * @override
     */
    apply(): Promise<void>;
    /**
     * Resolves the query parameters for retrieving the post, including its URI.
     *
     * @returns A promise that resolves to the query parameters.
     */
    queryParams(): Promise<StepPostQueryParams>;
}

/**
 * Represents a step for retrieving the profile of the author of a specific post.
 * Extends {@link StepActor} to reuse actor-related logic.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepPost}.
 * @typeParam C - The context type, defaulting to {@link StepPostOutput}.
 * @typeParam O - The output type, defaulting to {@link StepPostAuthorOutput}.
 * @public
 */
export declare class StepPostAuthor<P = StepPost, C extends StepPostOutput = StepPostOutput, O extends StepPostAuthorOutput = StepPostAuthorOutput> extends StepActor<P, C, O> {
    /**
     * Applies the step logic to retrieve the profile of the author of the post.
     *
     * @throws Error
     * If no context is available for the step, indicating the post is unknown.
     */
    apply(): Promise<void>;
}

/**
 * Represents the output of the `StepPostAuthor`, which is equivalent to {@link StepActorOutput}.
 * @public
 */
export declare type StepPostAuthorOutput = StepActorOutput;

/**
 * Represents a step for liking a specific post.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepPost}.
 * @typeParam C - The context type, defaulting to {@link StepPostOutput}.
 * @typeParam O - The output type, defaulting to {@link StepPostLikeOutput}.
 * @public
 */
export declare class StepPostLike<P = StepPost, C extends StepPostOutput = StepPostOutput, O extends StepPostLikeOutput = StepPostLikeOutput> extends Step<P, C, O> {
    /**
     * Applies the step logic to like the post provided in the context.
     * The resulting URI and CID of the like operation are stored in the output.
     *
     * @override
     * @throws Error
     * If the context does not provide the required `uri` and `cid`.
     */
    apply(): Promise<void>;
}

/**
 * Represents the output of the `StepPostLike` step, which contains the URI and CID of the liked post.
 * @public
 */
export declare type StepPostLikeOutput = {
    "uri": string;
    "cid": string;
};

/**
 * Represents the output of a retrieved post, including its URI, CID, record data, and author profile.
 */
export declare type StepPostOutput = {
    "uri": string;
    "cid": string;
    "record": object;
    "author": AppBskyActorDefs.ProfileViewBasic;
};

/**
 * Defines the query parameters for retrieving a post.
 * @public
 */
export declare type StepPostQueryParams = AppBskyFeedGetPosts.QueryParams;

/**
 * Represents a step for replying to a specific post.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepPost}.
 * @typeParam C - The context type, defaulting to {@link StepPostOutput}.
 * @typeParam O - The output type, defaulting to {@link StepPostReplyOutput}.
 * @public
 */
export declare class StepPostReply<P = StepPost, C extends StepPostOutput = StepPostOutput, O extends StepPostReplyOutput = StepPostReplyOutput> extends Step<P, C, O> {
    /**
     * The record containing the reply parameters, which can be resolved dynamically at runtime.
     */
    _record: ResolvablePostReplyParams;
    /**
     * Initializes the StepPostReply instance with the given agent, parent, and reply record parameters.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param record - The reply parameters, possibly resolvable at runtime.
     */
    constructor(agent: AtpAgent, parent: P, record: ResolvablePostReplyParams);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Applies the step logic to create a reply to the post provided in the context.
     * The resulting URI and CID of the reply are stored in the output.
     *
     * @override
     */
    apply(): Promise<void>;
    /**
     * Resolves the query parameters for the reply, including reply references and creation time.
     *
     * @returns A promise that resolves to the complete reply record.
     */
    queryParams(): Promise<AppBskyFeedPost.Record>;
    /**
     * Constructs the reply parameters for the post, including parent and root references.
     *
     * @returns An object containing the root and parent references for the reply.
     */
    replyParams(): AppBskyFeedPost.Record["reply"];
}

/**
 * Represents the output of a post reply step, including the URI and CID of the reply.
 * @public
 */
export declare type StepPostReplyOutput = StepPostReplyRecordRef;

/**
 * Represents a reference to a post record, including its URI and CID.
 * @public
 */
export declare interface StepPostReplyRecordRef {
    "uri": string;
    "cid": string;
}

/**
 * Represents the reply structure, containing references to both the parent and root posts.
 * @public
 */
export declare interface StepPostReplyRef {
    "parent": StepPostReplyRecordRef;
    "root": StepPostReplyRecordRef;
}

/**
 * Represents a step for reposting a specific post.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepPost}.
 * @typeParam C - The context type, defaulting to {@link StepPostOutput}.
 * @typeParam O - The output type, defaulting to {@link StepPostRepostOutput}.
 * @public
 */
export declare class StepPostRepost<P = StepPost, C extends StepPostOutput = StepPostOutput, O extends StepPostRepostOutput = StepPostRepostOutput> extends Step<P, C, O> {
    /**
     * Applies the step logic to repost the post provided in the context.
     * The resulting URI and CID of the repost are stored in the output.
     *
     * @override
     * @throws Error
     * If the context does not provide the required `uri` and `cid`.
     */
    apply(): Promise<void>;
}

/**
 * Represents the output of a repost step, including the URI and CID of the reposted content.
 * @public
 */
export declare type StepPostRepostOutput = {
    "uri": string;
    "cid": string;
};

/**
 * Abstract class representing a list of posts to process.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepBuilder}.
 * @typeParam C - The child context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to {@link StepPostsOutput}.
 * @public
 */
export declare class StepPosts<P = StepBuilder, C = null, O extends StepPostsOutput = StepPostsOutput> extends StepBuilderList<P, C, O> {
    /**
     * Holds the list of steps to be executed for each post entry.
     */
    _steps: StepPostsEntry<this>[];
    /**
     * The URIs of the posts to retrieve, which can be resolved dynamically at runtime.
     */
    _uris: Resolvable<StepPostsUris>;
    /**
     * Initializes the StepPost instance with the given agent, parent, and URI.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param uris - The URIs of the post sto retrieve, possibly resolvable at runtime.
     */
    constructor(agent: AtpAgent, parent: P, uris?: Resolvable<StepPostsUris>);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepPosts} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Appends a new post entry step to the current list and returns it.
     *
     * @param iterator - Optional iterator function to be executed for each item in the list.
     * @returns The newly appended {@link StepPostsEntry} instance.
     */
    each(iterator?: StepBuilderListIterator): StepPostsEntry<this>;
    /**
     * Applies the step by resolving the actor parameter and fetching the actors' profiles.
     * Sets the posts data as the output of this step.
     * @returns A promise that resolves when the step is complete.
     */
    applyPagination(): Promise<void>;
}

/**
 * Represents an individual entry step within a {@link StepPosts} list.
 * Extends {@link PostMixins} to include common post-related operations.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepPosts}.
 * @typeParam C - The context type, defaulting to {@link StepPostsOutput}.
 * @typeParam O - The output type, defaulting to `unknown`.
 * @public
 */
export declare class StepPostsEntry<P = StepPosts, C extends StepPostsOutput = StepPostsOutput, O = unknown> extends PostMixins<P, C, O> {
    /**
     * Applies the step's logic but do nothing by default. This method is
     * usually be overridden by child classes but will not throw an error if not.
     * @override
     */
    apply(): Promise<void>;
}

/**
 * Represents the output of a posts step, consisting of an array of post views.
 * @public
 */
export declare type StepPostsOutput = AppBskyFeedDefs.PostView[];

/**
 * Type for the parameter passed to the {@link StepPosts} class.
 * Represents the URIs of several post.
 * Bluesky currently limit this to 25 posts.
 * @public
 */
export declare type StepPostsUris = (string | AtUri)[];

/**
 * Represents a post's URI, which can be a string or an {@link AtUri}.
 * @public
 */
export declare type StepPostUri = string | AtUri;

/**
 * A step
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepBuilder}.
 * @typeParam C - The context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to `string`.
 * @public
 */
export declare class StepSave<P = StepBuilder, C = null, O = string> extends Step<P, C, O> {
    /**
     * Path of the JSON file to save the current output.
     */
    protected _path: StepSavePath;
    /**
     * Initializes the StepSave instance with the given path.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param path - The path of the JSON file to save the output. If not provided, the file path will be created using the current timestamp.
     */
    constructor(agent: AtpAgent, parent: P, path?: StepSavePath);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Applies the step logic to save the output to a JSON file.
     *
     * @override
     */
    apply(): Promise<void>;
}

/**
 * Type representing the path of the JSON file to save the output.
 * @public
 */
export declare type StepSavePath = string | null;

/**
 * Represents a step for searching posts on Bluesky, with support for pagination.
 *
 * @typeParam P - The parent type of this step.
 * @typeParam C - The child context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to {@link StepSearchPostsOutput}.
 * @public
 */
export declare class StepSearchPosts<P, C = null, O extends StepSearchPostsOutput = StepSearchPostsOutput> extends StepPosts<P, C, O> {
    /**
     * The initial query parameters for the search.
     */
    _queryParams: StepSearchPostsQueryParams;
    /**
     * Initializes the StepSearchPosts instance with the given agent, parent, and query parameters.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param queryParams - The initial query parameters for the search.
     */
    constructor(agent: AtpAgent, parent: P, queryParams: StepSearchPostsQueryParams);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Applies the pagination logic to retrieve posts based on the search parameters.
     * Results are stored in the `output` property.
     *
     * @override
     */
    applyPagination(): Promise<void>;
    /**
     * Constructs the query parameters for the current pagination step.
     *
     * @param cursor - The cursor indicating the current position in the paginated results.
     * @returns The query parameters for the search.
     */
    queryParams(cursor: StepSearchPostsQueryParamsCursor): StepSearchPostsQueryParams;
}

/**
 * Represents the output of a search posts step, consisting of an array of posts.
 * @public
 */
export declare type StepSearchPostsOutput = AppBskyFeedSearchPosts.OutputSchema["posts"];

/**
 * Represents the query parameters for searching posts.
 * @public
 */
export declare type StepSearchPostsQueryParams = AppBskyFeedSearchPosts.QueryParams;

/**
 * Represents the cursor for paginating through search post results.
 * @public
 */
export declare type StepSearchPostsQueryParamsCursor = StepSearchPostsQueryParams["cursor"] | undefined;

/**
 * @experimental
 *
 * Represents a streaming step that processes posts in real-time.
 *
 * @typeParam P - The type of the parent constraint.
 * @typeParam C - The type of the context.
 * @typeParam O - The type of the output, defaults to `StepPostOutput`.
 * @public
 *
 * @remarks
 * This class allows for the streaming and processing of posts as they are
 * committed to the network. To achieve this, it utilizes the Bluesky Firehose
 * API to listen for post events and process them in real-time. The stream can
 * use a lot of bandwidth since it processes every post event.
 *
 * @see {@link StepBuilderStream}
 */
export declare class StepStreamPosts<P, C = unknown, O = StepPostOutput> extends StepBuilderStream<P, C, O> {
    _steps: StepPostsEntry<this>[];
    /**
     * Adds a {@link StepPostsEntry} step to the stream.
     *
     * @returns The new {@link StepPostsEntry} instance.
     */
    each(): StepPostsEntry<this>;
    /**
     * Builds and returns an event emitter configured for post events.
     *
     * @returns A promise that resolves to the event emitter.
     */
    buildEventEmitter(): Promise<JetstreamEventEmitter>;
    /**
     * Resolves the output for a given commit message.
     *
     * @param message - The commit message containing post data.
     * @returns A promise that resolves to the output object.
     */
    resolveOutput(message: JetstreamMessageCommit): Promise<O>;
}

/**
 * Represents a step that allows interception of the current {@link Step} instance
 * for custom processing or side effects.
 *
 * @typeParam P - The type of the parent constraint.
 * @typeParam C - The type of the context.
 * @typeParam O - The type of the output, defaults to `null`.
 * @public
 *
 * @remarks
 * The `StepTap` class enables the execution of a provided interceptor function,
 * which receives the current step as its argument. This is useful for debugging,
 * logging, or performing side effects without altering the main flow.
 *
 * @see {@link Step}
 */
export declare class StepTap<P = StepBuilder, C = unknown, O = null> extends Step<P, C, O> {
    /**
     * The interceptor function to be executed during the apply phase.
     * It receives the current {@link Step} instance and can perform custom operations.
     */
    protected _interceptor: StepTapInterceptor;
    /**
     * Creates an instance of `StepTap`.
     *
     * @param agent - The ATP agent used for API interactions.
     * @param parent - The parent step or context.
     * @param interceptor - A function that intercepts the current step for custom processing.
     */
    constructor(agent: AtpAgent, parent: P, interceptor: StepTapInterceptor);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Executes the interceptor function with the current step as its argument.
     */
    apply(): Promise<void>;
}

/**
 * Defines the type for a function that intercepts a {@link Step} instance.
 * The interceptor can be synchronous or asynchronous.
 * @public
 */
export declare type StepTapInterceptor = ((step: Step) => unknown) | ((step: Step) => Promise<unknown>);

/**
 * Represents step that introduces a delay before proceeding to the next step.
 *
 * @typeParam P - The parent type of this step, defaulting to {@link StepBuilder}.
 * @typeParam C - The context type, defaulting to `null`.
 * @typeParam O - The output type, defaulting to `null`.
 * @public
 */
export declare class StepWait<P = StepBuilder, C = null, O = null> extends Step<P, C, O> {
    /**
     * The duration of the delay in milliseconds.
     */
    protected _duration: number;
    /**
     * Initializes the StepWait instance with the given duration.
     *
     * @param agent - The AT protocol agent used for API calls.
     * @param parent - The parent step in the chain.
     * @param duration - The duration of the delay in milliseconds. Defaults to `0`.
     */
    constructor(agent: AtpAgent, parent: P, duration?: number);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Applies the step logic to introduce a delay.
     * The process is paused for the duration specified in `_duration`.
     *
     * @override
     */
    apply(): Promise<void>;
}

/**
 * Represents a conditional step that evaluates a predicate to determine its outcome.
 *
 * @typeParam P - The type of the parent constraint.
 * @typeParam C - The type of the context.
 * @typeParam O - The type of the output, defaults to `boolean`.
 * @public
 *
 * @remarks
 * This step evaluates a predicate, which can be a boolean value or a resolvable that
 * resolves to a boolean. The result of the predicate evaluation is stored in the output.
 */
export declare class StepWhen<P = StepBuilder, C = unknown, O = boolean> extends Step<P, C, O> {
    private _predicate;
    /**
     * Creates an instance of `StepWhen`.
     *
     * @param agent - The ATP agent used for API interactions.
     * @param parent - The parent step or context.
     * @param predicate - A boolean or a resolvable that resolves to a boolean,
     *                    determining the condition for this step.
     */
    constructor(agent: AtpAgent, parent: P, predicate: Resolvable<StepWhenPredicate>);
    /**
     * Clones the current step and returns a new instance with the same parameters.
     * @param rest - Additional parameters to pass to the cloned step. This is useful for child class overriding the clone.
     * @returns A new {@link StepActor} instance.
     */
    clone(...rest: unknown[]): this;
    /**
     * Evaluates the predicate and sets the output to its result.
     */
    apply(): Promise<void>;
}

/**
 * Boolean values resolved by the {@link StepWhen} predicate.
 * @public
 */
export declare type StepWhenPredicate = boolean;

/**
 * Trotsky orchestrates a sequence of steps that interact with an {@link AtpAgent}.
 * @typeParam P - Type of the parent object.
 * @typeParam C - Type of the context object.
 * @typeParam O - Type of the output object.
 * @public
 */
declare class Trotsky extends StepBuilder {
    /**
     * Adds a {@link StepActor} step.
     * @param param - Parameters for the actor step.
     * @returns The new {@link StepActor} instance.
     */
    actor(param: Resolvable<StepActorParam>): StepActor<this>;
    /**
     * Adds a {@link StepActors} step.
     * @param param - Parameters for the actors step.
     * @returns The new {@link StepActors} instance.
     */
    actors(param: Resolvable<StepActorsParam>): StepActors<this>;
    /**
     * Adds a {@link StepPost} step.
     * @param uri - The post URI.
     * @returns The new {@link StepPost} instance.
     */
    post(uri: Resolvable<StepPostUri>): StepPost<this>;
    /**
     * Adds a {@link StepPosts} step.
     * @param uris - The post URIs.
     * @returns The new {@link StepPosts} instance.
     */
    posts(uris: Resolvable<StepPostsUris>): StepPosts<this>;
    /**
     * Adds a {@link StepCreatePost} step.
     * @param record - Parameters for creating a post.
     * @returns The new {@link StepCreatePost} instance.
     */
    createPost(record: StepCreatePostParams): StepCreatePost<this>;
    /**
     * Adds a {@link StepList} step.
     * @param uri - The list URI.
     * @returns The new {@link StepList} instance.
     */
    list(uri: Resolvable<StepListUri>): StepList<this, null, ListView>;
    /**
     * Adds a {@link StepSearchPosts} step.
     * @param queryParams - Search parameters.
     * @returns The new {@link StepSearchPosts} instance.
     */
    searchPosts(queryParams: QueryParams): StepSearchPosts<this>;
    /**
     * Adds a {@link StepStreamPosts} step.
     * @returns The new {@link StepStreamPosts} instance.
     */
    streamPosts<T = StepStreamPosts<this>>(): T;
    /**
     * Adds a {@link StepSave} step.
     * @param path - The path of the JSON file to save the output. If not provided, the file path will be created using the current step name.
     * @returns The current {@link Trotsky} instance.
     */
    save(path?: StepSavePath): this;
    /**
     * Adds a {@link StepWait} step.
     * @param duration - The duration to wait, in milliseconds.
     * @returns The current {@link Trotsky} instance.
     */
    wait(duration?: number): this;
    /**
     *Adds a {@link StepWhen} step.
     * @param predicate - The predicate function.
     * @returns The current {@link Trotsky} instance.
     */
    when(predicate: Resolvable<StepWhenPredicate>): this;
    /**
     * Adds a {@link StepTap} step.
     * @returns The current {@link Trotsky} instance.
     */
    tap(interceptor: StepTapInterceptor): this;
    /**
     * Initializes a new {@link Trotsky} instance with the specified agent.
     * @param agent - The {@link AtpAgent} instance.
     * @returns A new {@link Trotsky} instance.
     */
    static init(agent: AtpAgent): Trotsky;
}
export { Trotsky }
export default Trotsky;

export { }
