"use strict";
/**
 * This package provides a comprehensive suite of TypeScript classes and utilities designed
 * to facilitate interaction with the AT Protocol (ATProto) and Bluesky's social networking
 * services.
 *
 * It offers a structured approach to building and executing sequences of operations—referred
 * to as "steps"—that interact with various aspects of the protocol, such as reading user
 * profiles, posts, likes, follows, and more.
 *
 * By leveraging the schema definitions outlined in the ATProto lexicons,this package
 * ensures type safety and consistency when interfacing with ATProto's decentralized social
 * networking ecosystem.
 *
 * @remarks
 * Although all classes and methods are documented and public, the package is intended for
 * use mainly throught the {@link Trotsky} class, which serves as the main entry point using
 * a builder pattern to create and execute sequences of steps.
 *
 * @example
 * ```ts
 * import { AtpAgent } from "@atproto/api"
 * import { Trotsky } from "trotsky"
 *
 * async function main() {
 *  const agent = new AtpAgent({ service: "https://bsky.social" })
 *  await agent.login({ identifier: 'trotsky.pirhoo.com', password: 'p4ssw0rd' })
 *
 *  await Trotsky.init(agent)
 *    .searchPosts({ q: "pizza" })
 *      .take(3)
 *      .each()
 *        .reply({ text: "🍕 Pizza party! 🍕" })
 *        .wait(1000)
 *        .run()
 * }
 *
 * main()
 * ```
 *
 * @packageDocumentation
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
// Abstract and top-level classes
__exportStar(require("./core/StepBuilder"), exports);
__exportStar(require("./core/Trotsky"), exports);
__exportStar(require("./core/Step"), exports);
__exportStar(require("./core/StepBuilderList"), exports);
__exportStar(require("./core/StepBuilderListEntry"), exports);
__exportStar(require("./core/StepBuilderStream"), exports);
__exportStar(require("./core/StepBuilderStreamEntry"), exports);
// Mixins
__exportStar(require("./core/mixins/ActorMixins"), exports);
__exportStar(require("./core/mixins/ListMixins"), exports);
__exportStar(require("./core/mixins/PostMixins"), exports);
// Streams
__exportStar(require("./core/StepStreamPosts"), exports);
// List of actors
__exportStar(require("./core/StepActors"), exports);
__exportStar(require("./core/StepActorsEntry"), exports);
// List of posts
__exportStar(require("./core/StepPosts"), exports);
__exportStar(require("./core/StepPostsEntry"), exports);
__exportStar(require("./core/StepSearchPosts"), exports);
// List of lists
__exportStar(require("./core/StepLists"), exports);
__exportStar(require("./core/StepListsEntry"), exports);
// Single actor
__exportStar(require("./core/StepActor"), exports);
__exportStar(require("./core/StepActorBlock"), exports);
__exportStar(require("./core/StepActorFollow"), exports);
__exportStar(require("./core/StepActorFollowers"), exports);
__exportStar(require("./core/StepActorFollowings"), exports);
__exportStar(require("./core/StepActorLikes"), exports);
__exportStar(require("./core/StepActorLists"), exports);
__exportStar(require("./core/StepActorPosts"), exports);
__exportStar(require("./core/StepActorStreamPosts"), exports);
__exportStar(require("./core/StepActorUnblock"), exports);
__exportStar(require("./core/StepActorUnfollow"), exports);
// Single post
__exportStar(require("./core/StepCreatePost"), exports);
__exportStar(require("./core/StepPost"), exports);
__exportStar(require("./core/StepPostAuthor"), exports);
__exportStar(require("./core/StepPostLike"), exports);
__exportStar(require("./core/StepPostReply"), exports);
__exportStar(require("./core/StepPostRepost"), exports);
// Single lists
__exportStar(require("./core/StepList"), exports);
__exportStar(require("./core/StepListMembers"), exports);
// Utils
__exportStar(require("./core/StepTap"), exports);
__exportStar(require("./core/StepWait"), exports);
__exportStar(require("./core/StepWhen"), exports);
__exportStar(require("./core/StepSave"), exports);
// Main entry point class
var Trotsky_1 = require("./core/Trotsky");
Object.defineProperty(exports, "default", { enumerable: true, get: function () { return Trotsky_1.Trotsky; } });
//# sourceMappingURL=trotsky.js.map